import re
import requests

from coalib.results.Diff import Diff
from coalib.bears.LocalBear import LocalBear
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY
from coalib.results.Result import Result


class InvalidLinkBear(LocalBear):
    DEFAULT_TIMEOUT = 2

    @classmethod
    def check_prerequisites(cls):
        code = cls.get_status_code_or_error(
            "http://www.google.com", cls.DEFAULT_TIMEOUT)
        return ("You are not connected to the internet."
                if code is None else True)

    @staticmethod
    def get_status_code_or_error(url, timeout):
        try:
            code = requests.head(url, allow_redirects=False,
                                 timeout=timeout).status_code
            return code
        except requests.exceptions.RequestException:
            pass

    @staticmethod
    def find_links_in_file(file, timeout):
        regex = re.compile(
            r'((ftp|http)s?:\/\/\S+\.(?:[^\s\(\)\'\>\|]+|'
            r'\([^\s\(\)]*\))*)(?<!\.)(?<!\,)')
        for line_number, line in enumerate(file):
            match = regex.search(line)
            if match:
                link = match.group()
                code = InvalidLinkBear.get_status_code_or_error(link, timeout)
                yield line_number + 1, link, code

    def run(self, filename, file, timeout: int=DEFAULT_TIMEOUT):
        '''
        Yields results for all invalid links in a file.

        :param timeout: Request timeout period.
        '''
        for line_number, link, code in InvalidLinkBear.find_links_in_file(
                file, timeout):
            if code is None:
                yield Result.from_values(
                    origin=self,
                    message=('Broken link - unable to connect to '
                             '{url}').format(url=link),
                    file=filename,
                    line=line_number,
                    severity=RESULT_SEVERITY.MAJOR)
            elif not 200 <= code < 300:
                if 400 <= code < 600:  # HTTP status 40x or 50x
                    yield Result.from_values(
                        origin=self,
                        message=('Broken link - unable to connect to {url} '
                                 '(HTTP Error: {code})'
                                 ).format(url=link, code=code),
                        file=filename,
                        line=line_number,
                        severity=RESULT_SEVERITY.NORMAL)
                if 300 <= code < 400:  # HTTP status 30x
                    redirect_url = requests.head(link, allow_redirects=True).url
                    diff = Diff(file)
                    current_line = file[line_number - 1]
                    start = current_line.find(link)
                    end = start + len(link)
                    replacement = current_line[:start] + \
                        redirect_url + current_line[end:]
                    diff.change_line(line_number, current_line, replacement)

                    yield Result.from_values(
                        self,
                        'This link redirects to ' + redirect_url,
                        diffs={filename: diff},
                        file=filename,
                        line=line_number,
                        severity=RESULT_SEVERITY.NORMAL)



import os
from queue import Queue
from shutil import which
from unittest.case import skipIf

from bears.c_languages.CSharpLintBear import CSharpLintBear
from tests.LocalBearTestHelper import LocalBearTestHelper
from coalib.settings.Section import Section


@skipIf(which('mcs') is None, 'mono mcs is not installed')
class CSharpLintBearTest(LocalBearTestHelper):

    def setUp(self):
        self.section = Section("test section")
        self.uut = CSharpLintBear(self.section, Queue())
        self.test_file1 = os.path.join(os.path.dirname(__file__),
                                       "test_files",
                                       "csharplint_test1.cs")
        self.test_file2 = os.path.join(os.path.dirname(__file__),
                                       "test_files",
                                       "csharplint_test2.cs")

    def test_run(self):
        # Test a file with no issues
        self.check_validity(self.uut, [], self.test_file1)

        # Test a file with issues
        self.check_validity(self.uut, [], self.test_file2, valid=False)



import json

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.misc.Shell import escape_path_argument
from coalib.results.Result import Result
from coalib.settings.Setting import path


class TSLintBear(LocalBear, Lint):
    executable = 'tslint'

    def process_output(self, output, filename, file):
        output = json.loads("".join(output)) if output else []
        for issue in output:
            yield Result.from_values(
                origin="{} ({})".format(self.__class__.__name__,
                                        issue['ruleName']),
                message=issue["failure"],
                file=issue["name"],
                line=int(issue["startPosition"]["line"]) + 1,
                end_line=int(issue["endPosition"]["line"]) + 1,
                column=int(issue["startPosition"]["character"]) + 1,
                end_column=int(issue["endPosition"]["character"]) + 1)

    def run(self,
            filename,
            file,
            tslint_config: path="",
            rules_dir: path=""):
        '''
        Checks the code with ``tslint`` on each file separately.

        :param tslint_config: Path to configuration file.
        :param rules_dir:     Rules directory
        '''
        self.arguments = "--format json"
        if tslint_config:
            self.arguments += (" --config " +
                               escape_path_argument(tslint_config))
        if rules_dir:
            self.arguments += (" --rules-dir " +
                               escape_path_argument(rules_dir))
        self.arguments += " {filename}"
        return self.lint(filename, file)



from bears.html.HTMLLintBear import HTMLLintBear
from tests.LocalBearTestHelper import verify_local_bear

test_file = """
<html>
  <body>
    <h1>Hello, world!</h1>
  </body>
</html>
""".splitlines(keepends=True)

HTMLLintBearTest = verify_local_bear(HTMLLintBear,
                                     valid_files=(),
                                     invalid_files=(test_file,),
                                     tempfile_kwargs={"suffix": ".html"})

HTMLLintBearIgnoreTest = verify_local_bear(
    HTMLLintBear,
    valid_files=(test_file,),
    invalid_files=(),
    settings={'htmllint_ignore': 'optional_tag'},
    tempfile_kwargs={"suffix": ".html"})

HTMLLintBearIgnoreQuotationTest = verify_local_bear(
    HTMLLintBear,
    valid_files=(),
    invalid_files=(test_file,),
    settings={'htmllint_ignore': 'quotation'},
    tempfile_kwargs={"suffix": ".html"})



from coalib.bears.LocalBear import LocalBear
from coalib.results.Result import Result
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY


class LineCountBear(LocalBear):

    def run(self, filename, file, *args):
        '''
        Counts the lines of each file.
        '''
        yield Result.from_values(
            origin=self,
            message="This file has {count} lines.".format(count=len(file)),
            severity=RESULT_SEVERITY.INFO,
            file=filename)



import os

from bears.typescript.TSLintBear import TSLintBear
from tests.LocalBearTestHelper import verify_local_bear

good_file = """function findTitle(title) {
    let titleElement = "hello";
    return title;
}
let t = findTitle("mytitle");
t.innerHTML = "New title";
""".splitlines(True)

bad_file = """function findTitle(title) {
    let titleElement = 'hello';
    return title;
}
let t = findTitle('mytitle');
t.innerHTML = 'New title';
""".splitlines(True)

tslintconfig = os.path.join(os.path.dirname(__file__),
                            "test_files",
                            "tslint.json")

TSLintBearWithoutConfig = verify_local_bear(TSLintBear,
                                            valid_files=(good_file,),
                                            invalid_files=(bad_file,),
                                            tempfile_kwargs={"suffix": ".ts"})

TSLintBearTestWithConfig = verify_local_bear(TSLintBear,
                                             valid_files=(bad_file,),
                                             invalid_files=(good_file,),
                                             settings={"tslint_config":
                                                       tslintconfig},
                                             tempfile_kwargs={"suffix": ".ts"})

TSLintBearOtherOptions = verify_local_bear(TSLintBear,
                                           valid_files=(good_file,),
                                           invalid_files=(bad_file,),
                                           settings={"rules_dir": "/"},
                                           tempfile_kwargs={"suffix": ".ts"})



from bears.sql.SQLintBear import SQLintBear
from tests.LocalBearTestHelper import verify_local_bear

good_file = """
SELECT * FROM table_name;
""".splitlines(True)

bad_file = """
invalid_SELECT 1;
""".splitlines(True)


SQLintBearTest = verify_local_bear(SQLintBear,
                                   valid_files=(good_file,),
                                   invalid_files=(bad_file,))



from bears.natural_language.ProseLintBear import ProseLintBear
from tests.LocalBearTestHelper import verify_local_bear

good_file = ["The 50s were swell."]
bad_file = ["The 50's were swell."]


ProseLintBearTest = verify_local_bear(ProseLintBear,
                                      valid_files=(good_file,),
                                      invalid_files=(bad_file,))



from bears.configfiles.DockerfileLintBear import DockerfileLintBear
from tests.LocalBearTestHelper import verify_local_bear

good_file = """
FROM ubuntu:14.04

# Install basic tools
RUN apt-get -y -qq update
RUN apt-get -y -qq upgrade
""".splitlines(keepends=True)


bad_file = """
FROM ubuntu:14.04

# Install basic tools
apt-get -y -qq update
apt-get -y -qq upgrade
""".splitlines(keepends=True)


DockerfileLintBearTest = verify_local_bear(DockerfileLintBear,
                                           valid_files=(good_file,),
                                           invalid_files=(bad_file,))



import re

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY


class SCSSLintBear(LocalBear, Lint):
    executable = 'scss-lint'
    arguments = '{filename}'
    output_regex = re.compile(
                r'(?P<file_name>.+):(?P<line>\d+)\s*'
                r'(\[(?P<severity>.*?)\])\s*'
                r'(?P<message>.*)')
    severity_map = {
        "I": RESULT_SEVERITY.INFO,
        "W": RESULT_SEVERITY.NORMAL,
        "E": RESULT_SEVERITY.MAJOR
    }

    def run(self, filename, file):
        '''
        Checks the code with `scss-lint` on each file separately.
        '''
        return self.lint(filename)



import json

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.results.Result import Result


class JSComplexityBear(LocalBear, Lint):
    executable = "cr"

    def process_output(self, output, filename, file):
        message = "{} has a cyclomatic complexity of {}."
        if output:
            output = json.loads("".join(output))
            for function in output["reports"][0]["functions"]:
                if function["cyclomatic"] >= self.cc_threshold:
                    yield Result.from_values(
                        origin=self,
                        message=message.format(function["name"],
                                               function["cyclomatic"]),
                        file=filename,
                        line=function["line"])

    def run(self,
            filename,
            file,
            cc_threshold: int=10):
        """
        Calculates cyclomatic complexity using ``cr``.

        :param cc_threshold: Threshold value for cyclomatic complexity
        """
        self.cc_threshold = cc_threshold
        self.arguments = "--format json {filename}"
        return self.lint(filename, file)



import json

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.results.Diff import Diff
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY
from coalib.results.Result import Result


class RuboCopBear(LocalBear, Lint):
    executable = 'rubocop'
    # Need both stdin and filename. Explained in the comment:
    # https://github.com/bbatsov/rubocop/pull/2146#issuecomment-131403694
    arguments = '{filename} --stdin --format=json'
    severity_map = {
        "error": RESULT_SEVERITY.MAJOR,
        "warning": RESULT_SEVERITY.NORMAL,
        "convention": RESULT_SEVERITY.INFO
    }
    use_stdin = True

    def run(self, filename, file):
        '''
        Checks the code with ``rubocop``. This will run ``rubocop``
        over each of the files separately.
        '''
        return self.lint(filename, file)

    def _process_issues(self, output, filename):
        output = json.loads("".join(output))
        assert len(output['files']) == 1
        for result in output['files'][0]['offenses']:
            # TODO: Add condition for auto-correct, when rubocop is updated.
            # Relevant Issue: https://github.com/bbatsov/rubocop/issues/2932
            yield Result.from_values(
                origin="{class_name} ({rule})".format(
                    class_name=self.__class__.__name__,
                    rule=result['cop_name']),
                message=result['message'],
                file=filename,
                diffs=None,
                severity=self.severity_map[result['severity']],
                line=result['location']['line'],
                column=result['location']['column'],
                # Tested with linebreaks, it's secure.
                end_column=result['location']['column'] +
                result['location']['length'])



from bears.python.PyDocStyleBear import PyDocStyleBear
from tests.LocalBearTestHelper import verify_local_bear


good_file = '''
"""example module level docstring."""


def hello():
    """Print hello world."""
    print("hello world")

'''.splitlines(keepends=True)


bad_file = '''
"""
example module level docstring
"""

def hello():
    print("hello world")

'''.splitlines(keepends=True)

PyDocStyleBearTest = verify_local_bear(PyDocStyleBear,
                                       valid_files=(good_file,),
                                       invalid_files=(bad_file,),
                                       tempfile_kwargs={"suffix": ".py"})



from bears.general.LineLengthBear import LineLengthBear
from tests.LocalBearTestHelper import verify_local_bear

test_file = """
test
too
er
e
""".splitlines(keepends=True)


LineLengthBearTest = verify_local_bear(LineLengthBear,
                                       valid_files=(test_file,),
                                       invalid_files=(["testa"],
                                                      ["test line"]),
                                       settings={"max_line_length": "4"})


LineLengthBearIgnoreRegexTest = verify_local_bear(
    LineLengthBear,
    valid_files=(test_file,
                 ["http://a.domain.de"],
                 ["ftp://a.domain.de"],
                 ["hi there ftp://!"]),
    invalid_files=(["http not a link"],),
    settings={
        "max_line_length": "4",
        "ignore_length_regex": "http://, https://, ftp://"})



from bears.c_languages.CPPLintBear import CPPLintBear
from tests.LocalBearTestHelper import verify_local_bear

test_file = """
int main() {
    return 0;
}
""".splitlines(keepends=True)

CPPLintBearTest = verify_local_bear(CPPLintBear,
                                    valid_files=(),
                                    invalid_files=(test_file,),
                                    tempfile_kwargs={"suffix": ".cpp"})

CPPLintBearIgnoreConfigTest = verify_local_bear(
    CPPLintBear,
    valid_files=(test_file,),
    invalid_files=(),
    settings={'cpplint_ignore': 'legal'},
    tempfile_kwargs={"suffix": ".cpp"})

CPPLintBearLineLengthConfigTest = verify_local_bear(
    CPPLintBear,
    valid_files=(),
    invalid_files=(test_file,),
    settings={'cpplint_ignore': 'legal',
              'max_line_length': '13'},
    tempfile_kwargs={"suffix": ".cpp"})



import re

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY


class SQLintBear(LocalBear, Lint):
    executable = 'sqlint'
    output_regex = re.compile(
        r'(?P<file_name>.+?):(?P<line>\d+):(?P<column>\d+):'
        r'(?P<severity>ERROR|WARNING) (?P<message>.+(?:\r?\n  .+)*)')
    use_stdin = True
    severity_map = {
        "WARNING": RESULT_SEVERITY.NORMAL,
        "ERROR": RESULT_SEVERITY.MAJOR
    }

    def run(self, filename, file):
        '''
        Checks the given file using `sqlint`.
        '''
        return self.lint(filename, file)



import re

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.misc.Shell import escape_path_argument
from coalib.settings.Setting import path


class CMakeLintBear(LocalBear, Lint):
    executable = 'cmakelint'
    output_regex = re.compile(
            r'(?P<file_name>\S+):(?P<line>[0-9]+): (?P<message>.*)')

    def run(self, filename, file, cmakelint_config: path=""):
        '''
        Checks the code with ``cmakelint``.

        :param cmakelint_config: The location of the cmakelintrc config file.
        '''
        self.arguments = ""
        if cmakelint_config:
            self.arguments += (' --config=' +
                               escape_path_argument(cmakelint_config))

        self.arguments += ' {filename}'
        return self.lint(filename)



import re

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY


class CSSLintBear(LocalBear, Lint):
    executable = 'csslint'
    arguments = '--format=compact {filename}'
    output_regex = re.compile(
        r'(?P<file_name>.+):\s*'
        r'(?:line (?P<line>\d+), col (?P<col>\d+), )?'
        r'(?P<severity>Error|Warning) - (?P<message>.*)')
    severity_map = {
        "Error": RESULT_SEVERITY.MAJOR,
        "Warning": RESULT_SEVERITY.NORMAL}

    def run(self, filename, file):
        '''
        Checks the code with `csslint`.
        '''
        return self.lint(filename)



import os
from queue import Queue
from shutil import which
from unittest.case import skipIf

from bears.php.PHPLintBear import PHPLintBear
from tests.LocalBearTestHelper import LocalBearTestHelper
from coalib.settings.Section import Section


@skipIf(which('php') is None, 'PHP is not installed')
class PHPLintBearTest(LocalBearTestHelper):

    def setUp(self):
        self.section = Section("test section")
        self.uut = PHPLintBear(self.section, Queue())
        self.test_file1 = os.path.join(os.path.dirname(__file__),
                                       "test_files",
                                       "phplint_test1.php")
        self.test_file2 = os.path.join(os.path.dirname(__file__),
                                       "test_files",
                                       "phplint_test2.php")

    def test_run(self):
        # Test a file with errors and warnings
        self.check_validity(
            self.uut,
            [],
            self.test_file1,
            valid=False)

        # Test a file without any issues
        self.check_validity(
            self.uut,
            [],
            self.test_file2)



import os
import unittest
from queue import Queue

from clang.cindex import TranslationUnitLoadError

from bears.c_languages.codeclone_detection.ClangASTPrintBear import (
    ClangASTPrintBear)
from tests.BearTestHelper import generate_skip_decorator
from coalib.settings.Section import Section


@generate_skip_decorator(ClangASTPrintBear)
class ClangASTPrintBearTest(unittest.TestCase):

    def setUp(self):
        testfile = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                "sample.c"))
        self.queue = Queue()
        with open(testfile) as file:
            testfile_lines = file.readlines()
        self.uut = ClangASTPrintBear({testfile: testfile_lines},
                                     Section("name"),
                                     self.queue)

    def test_run(self):
        self.uut.run()
        with self.assertRaises(TranslationUnitLoadError):
            old_file_dict = self.uut.file_dict
            self.uut.file_dict = {"notexistant": []}
            self.uut.run()
            self.uut.file_dict = old_file_dict

    def test_ast(self):
        expected_ast = (
            """
|-stdio.h CursorKind.INCLUSION_DIRECTIVE Lines 2-2 (# include < stdio . h > #)
|-not_existant.c CursorKind.INCLUSION_DIRECTIVE Lines 3-3 (# include """
            """"not_existant.c" // Empty function)
|-test() CursorKind.FUNCTION_DECL Lines 6-6 (int test ( void ) ;)
|-g CursorKind.VAR_DECL Lines 9-9 (int g ;)
`-main(int, char *) CursorKind.FUNCTION_DECL Lines 12-30 (int main ( """
            """int t , char * args ) { // Usage in a call smile ( t , g ) ; """
            """// Simple stupid assignment t = g ; // Local declaration int """
            """* asd ; // Simple more stupid reassignment, this time using """
            """other syntax elems t = args [ g ] ; // Declaration in for """
            """loop for ( int i ; i < 5 ; i ++ ) { // Checking out constants"""
            """ printf ( "i is %d" , i ) ; } })\n""")

        self.uut.run()

        ast = "\n"
        # Only check beginning of AST
        for i in range(expected_ast.count("\n")-1):
            ast += self.queue.get(timeout=0).message + "\n"

        self.assertEqual(ast, expected_ast)



import re

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY


class CSharpLintBear(LocalBear, Lint):
    executable = 'mcs'
    output_regex = re.compile(
        r'(?P<filename>.+\.cs)\((?P<line>\d+),(?P<col>\d+)\): '
        r'(?P<severity>error|warning) (?P<severity_code>\w+): (?P<message>.+)'
    )
    use_stderr = True
    severity_map = {
        "warning": RESULT_SEVERITY.NORMAL,
        "error": RESULT_SEVERITY.MAJOR}
    arguments = "{filename}"

    def run(self, filename, file):
        '''
        Checks the code with `mcs` on each file separately.
        '''
        return self.lint(filename)



from queue import Queue

from bears.python.PyCommentedCodeBear import PyCommentedCodeBear
from tests.LocalBearTestHelper import LocalBearTestHelper
from coalib.settings.Section import Section


class PyCommentedCodeBearTest(LocalBearTestHelper):

    def setUp(self):
        self.uut = PyCommentedCodeBear(Section('name'), Queue())

    def test_valid(self):
        self.check_validity(self.uut, ["import sys"])
        self.check_validity(self.uut, ["a = 1 + 1"])
        self.check_validity(self.uut, ["# hey man!"])
        self.check_validity(self.uut, ['"""',
                                       'Hey, this is a code sample:',
                                       '>>> import os',
                                       '',
                                       'And when you use it you can simply '
                                       'do: `import os`.',
                                       '"""'])

    def test_invalid(self):
        self.check_validity(self.uut, ["# import os"], valid=False)
        self.check_validity(self.uut, ["# print('comment')"], valid=False)



import re
import shutil
import os

from coalib.bears.GlobalBear import GlobalBear
from coalib.misc.Shell import run_shell_command
from coalib.results.Result import Result


class GitCommitBear(GlobalBear):
    _git_command = "git log -1 --pretty=%B"

    @classmethod
    def check_prerequisites(cls):
        if shutil.which("git") is None:
            return "git is not installed."
        else:
            return True

    def run(self,
            shortlog_length: int=50,
            body_line_length: int=73,
            force_body: bool=False,
            allow_empty_commit_message: bool=False,
            shortlog_regex: str="",
            shortlog_trailing_period: bool=None):
        """
        Checks the current git commit message at HEAD.

        This bear ensures that the shortlog and body do not exceed a given
        line-length and that a newline lies between them.

        :param shortlog_length:            The maximum length of the shortlog.
                                           The shortlog is the first line of
                                           the commit message. The newline
                                           character at end does not count to
                                           the length.
        :param body_line_length:           The maximum line-length of the body.
                                           The newline character at each line
                                           end does not count to the length.
        :param force_body:                 Whether a body shall exist or not.
        :param allow_empty_commit_message: Whether empty commit messages are
                                           allowed or not.
        :param shortlog_regex:             A regex to check the shortlog with.
                                           A full match of this regex is then
                                           required. Passing an empty string
                                           disable the regex-check.
        :param shortlog_trailing_period:   Whether a dot shall be enforced at
                                           the end of the shortlog line.
                                           Providing ``None`` means
                                           "doesn't care".
        """
        config_dir = self.get_config_dir()
        old_dir = os.getcwd()
        if config_dir:
            os.chdir(config_dir)
        stdout, stderr = run_shell_command(self._git_command)

        if stderr:
            self.err("git:", repr(stderr))
            return

        # git automatically removes trailing whitespaces. Also we need to
        # remove the last \n printed to align the prompt onto the next line.
        stdout = stdout.splitlines()[:-1]

        if len(stdout) == 0:
            if not allow_empty_commit_message:
                yield Result(self, "HEAD commit has no message.")
            return

        yield from self.check_shortlog(shortlog_length,
                                       shortlog_regex,
                                       shortlog_trailing_period,
                                       stdout[0])
        yield from self.check_body(body_line_length, force_body, stdout[1:])

        os.chdir(old_dir)

    def check_shortlog(self,
                       shortlog_length,
                       regex,
                       shortlog_trailing_period,
                       shortlog):
        """
        Checks the given shortlog.

        :param shortlog_length:          The maximum length of the shortlog.
                                         The newline character at end does not
                                         count to the length.
        :param regex:                    A regex to check the shortlog with.
        :param shortlog_trailing_period: Whether a dot shall be enforced at end
                                         end or not (or ``None`` for "don't
                                         care").
        :param shortlog:                 The shortlog message string.
        """
        if len(shortlog) > shortlog_length:
            yield Result(self, "Shortlog of HEAD commit is too long.")

        if (shortlog[-1] != ".") == shortlog_trailing_period:
            yield Result(self,
                         "Shortlog of HEAD commit contains no period at end."
                         if shortlog_trailing_period else
                         "Shortlog of HEAD commit contains a period at end.")

        if regex != "":
            match = re.match(regex, shortlog)
            # fullmatch() inside re-module exists sadly since 3.4, but we
            # support 3.3 so we need to check that the regex matched completely
            # ourselves.
            if not match or match.end() != len(shortlog):
                yield Result(
                    self,
                    "Shortlog of HEAD commit does not match given regex.")

    def check_body(self, body_line_length, force_body, body):
        """
        Checks the given commit body.

        :param body_line_length: The maximum line-length of the body. The
                                 newline character at each line end does not
                                 count to the length.
        :param force_body:       Whether a body shall exist or not.
        :param body:             The commit body splitted by lines.
        """
        if len(body) == 0:
            if force_body:
                yield Result(self, "No commit message body at HEAD.")
            return

        if body[0] != "":
            yield Result(self, "No newline between shortlog and body at HEAD.")
            return

        if any(len(line) > body_line_length for line in body[1:]):
            yield Result(self, "Body of HEAD commit contains too long lines.")



from bears.c_languages.CPPCleanBear import CPPCleanBear
from tests.LocalBearTestHelper import verify_local_bear

good_file = """
int main() {
    return 0;
}
""".splitlines(keepends=True)


bad_file = """
int global_var = 3;

int main() {
    return 0;
}
""".splitlines(keepends=True)


CPPCleanBearTest = verify_local_bear(CPPCleanBear,
                                     valid_files=(good_file,),
                                     invalid_files=(bad_file,))



from bears.lua.LuaLintBear import LuaLintBear
from tests.LocalBearTestHelper import verify_local_bear


good_file = """
print("Hello World!")
""".splitlines(keepends=True)


bad_file = """
function factorial(n)
  local x = 1
  for i = 2, n do;
    x = x * i
  end
  return x
end
""".splitlines(keepends=True)


LuaLintBearTest = verify_local_bear(LuaLintBear,
                                    valid_files=(good_file,),
                                    invalid_files=(bad_file,))



from bears.coffee_script.CoffeeLintBear import CoffeeLintBear
from tests.LocalBearTestHelper import verify_local_bear

good_file = """
# Lint your CoffeeScript!

class Gangster

  wasItAGoodDay : () ->
    yes
""".splitlines(keepends=True)


warning_file = """
# Nested string interpolation
str = "Book by #{"#{firstName} #{lastName}".toUpperCase()}"
""".splitlines(keepends=True)


error_file = """
# Wrong capitalization
class theGangster

  wasItAGoodDay : () ->
    yes
""".splitlines(keepends=True)


invalid_file = """
# Coffeelint is buggy here and will generate an error with invalid CSV on this
var test
""".splitlines(keepends=True)


CoffeeLintBearTest = verify_local_bear(CoffeeLintBear,
                                       valid_files=(good_file,),
                                       invalid_files=(warning_file,
                                                      error_file,
                                                      invalid_file))



from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear


class GoImportsBear(Lint, LocalBear):
    executable = 'goimports'
    diff_message = "Imports need to be added/removed."
    use_stdin = True
    gives_corrected = True

    def run(self, filename, file):
        """
        Adds/Removes imports to Go code for missing imports.
        """
        return self.lint(filename, file)



import json

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.results.Result import Result
from coalib.results.Diff import Diff
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY


class HaskellLintBear(LocalBear, Lint):
    executable = 'hlint'
    arguments = '--json {filename}'
    severity_map = {
        "Error": RESULT_SEVERITY.MAJOR,
        "Warning": RESULT_SEVERITY.NORMAL,
        "Suggestion": RESULT_SEVERITY.INFO}
    gives_corrected = True

    def run(self, filename, file):
        '''
        Checks the given file with hlint.
        '''
        return self.lint(filename=filename, file=file)

    def _process_corrected(self, output, filename, file):
        output = json.loads("".join(output))

        for issue in output:
            assert issue["startLine"] == issue["endLine"]
            diff = Diff(file)
            line_nr = issue["startLine"]
            line_to_change = file[line_nr-1]
            newline = line_to_change.replace(issue["from"], issue["to"])
            diff.change_line(line_nr, line_to_change, newline)

            yield Result.from_values(
                origin=self,
                message=issue["hint"],
                file=filename,
                severity=self.severity_map[issue["severity"]],
                line=issue["startLine"],
                diffs={filename: diff})



from bears.java.InferBear import InferBear
from tests.LocalBearTestHelper import verify_local_bear


good_file = """
class InferGood {
    int test() {
        String s = null;
        return s == null ? 0 : s.length();
    }
}
""".splitlines(keepends=True)

bad_file = """
class InferBad {
    int test() {
        String s = null;
        return s.length();
    }
}
""".splitlines(keepends=True)


InferBearTest = verify_local_bear(InferBear,
                                  valid_files=(good_file,),
                                  invalid_files=(bad_file,),
                                  tempfile_kwargs={"suffix": ".java"})



import os

from bears.xml.XMLBear import XMLBear
from tests.LocalBearTestHelper import verify_local_bear


valid_xml_file = """<?xml version="1.0"?>
<a/>
""".splitlines(keepends=True)

invalid_xml_file = """
<a>blah</a>
""".splitlines(keepends=True)

invalid_xml_chars = """<?xml version="1.0"?>
<a>hey & hi</a>
""".splitlines(keepends=True)

valid_xml_chars = """<?xml version="1.0"?>
<a>hey and hi</a>
""".splitlines(keepends=True)

dtd_file = os.path.join(os.path.dirname(__file__),
                        "test_files",
                        "note.dtd")

schema_file = os.path.join(os.path.dirname(__file__),
                           "test_files",
                           "note.xsd")

valid_xml_path = list(open(os.path.join(
    os.path.dirname(__file__),
    "test_files",
    "note.xml"), 'r'))

valid_xml_url = list(open(os.path.join(
    os.path.dirname(__file__),
    "test_files",
    "concept-valid.xml"), 'r'))

invalid_xml_schema = list(open(os.path.join(
    os.path.dirname(__file__),
    "test_files",
    "xsd-error.xml"), 'r'))

invalid_xml_dtd = list(open(os.path.join(
    os.path.dirname(__file__),
    "test_files",
    "dtd-error.xml"), 'r'))

invalid_xml_url = list(open(os.path.join(
    os.path.dirname(__file__),
    "test_files",
    "concept-invalid.xml"), 'r'))

dtd_url = "http://docs.oasis-open.org/dita/v1.0.1/dtd/concept.dtd"

XMLBearCorrectedTest = verify_local_bear(
    XMLBear,
    valid_files=(valid_xml_file, valid_xml_chars),
    invalid_files=(invalid_xml_file, invalid_xml_chars),
    tempfile_kwargs={"suffix": ".xml"})

XMLBearSchemaTest = verify_local_bear(
    XMLBear,
    valid_files=(valid_xml_path,),
    invalid_files=(invalid_xml_schema,),
    settings={'xml_schema': schema_file},
    tempfile_kwargs={"suffix": ".xml"})

XMLBearDTDPathTest = verify_local_bear(
    XMLBear,
    valid_files=(valid_xml_path,),
    invalid_files=(invalid_xml_dtd,),
    settings={'xml_dtd': dtd_file},
    tempfile_kwargs={"suffix": ".xml"})

XMLBearDTDUrlTest = verify_local_bear(
    XMLBear,
    valid_files=(valid_xml_url,),
    invalid_files=(invalid_xml_url,),
    settings={'xml_dtd': dtd_url},
    tempfile_kwargs={"suffix": ".xml"})



from queue import Queue

from bears.general.SpaceConsistencyBear import (
    SpaceConsistencyBear, SpacingHelper)
from tests.LocalBearTestHelper import LocalBearTestHelper
from coalib.settings.Section import Section
from coalib.settings.Setting import Setting


class SpaceConsistencyBearTest(LocalBearTestHelper):

    def setUp(self):
        self.section = Section("test section")
        self.uut = SpaceConsistencyBear(self.section, Queue())

    def test_needed_settings(self):
        self.section.append(Setting("use_spaces", "true"))

        needed_settings = self.uut.get_non_optional_settings()
        self.assertEqual(len(needed_settings),
                         1 + len(SpacingHelper.get_non_optional_settings()))
        self.assertIn("use_spaces", needed_settings)

    def test_defaults(self):
        # use_spaces is no default, need to set it explicitly.
        self.section.append(Setting("use_spaces", "true"))

        self.check_validity(self.uut, ["    t"])
        self.check_validity(self.uut, ["\tt"], valid=False)
        self.check_validity(self.uut, ["t \n"], valid=False)
        self.check_validity(self.uut, ["t"],
                            force_linebreaks=False, valid=False)

    def test_data_sets_spaces(self):
        self.section.append(Setting("use_spaces", "true"))
        self.section.append(Setting("allow_trailing_whitespace", "false"))
        self.section.append(Setting("enforce_newline_at_EOF", "false"))

        self.check_validity(self.uut, ["    t"])
        self.check_validity(self.uut, ["t \n"], valid=False)
        self.check_validity(self.uut, ["\tt\n"], valid=False)

    def test_data_sets_tabs(self):
        self.section.append(Setting("use_spaces", "false"))
        self.section.append(Setting("allow_trailing_whitespace", "true"))
        self.section.append(Setting("enforce_newline_at_EOF", "false"))

        self.check_validity(self.uut, ["    t"], valid=False)
        self.check_validity(self.uut, ["t \n"])
        self.check_validity(self.uut, ["\tt\n"])

    def test_enforce_newline_at_eof(self):
        self.section.append(Setting("use_spaces", "true"))
        self.section.append(Setting("allow_trailing_whitespace", "true"))
        self.section.append(Setting("enforce_newline_at_EOF", "true"))

        self.check_validity(self.uut,
                            ["hello world  \n"],
                            force_linebreaks=False)
        self.check_validity(self.uut,
                            ["def somecode():\n",
                             "    print('funny')\n",
                             "    print('funny end.')\n"],
                            force_linebreaks=False)
        self.check_validity(self.uut,
                            [" no hello world"],
                            force_linebreaks=False,
                            valid=False)
        self.check_validity(self.uut,
                            ["def unfunny_code():\n",
                             "    print('funny')\n",
                             "    print('the result is not funny...')"],
                            force_linebreaks=False,
                            valid=False)



import re

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear


class VintBear(LocalBear, Lint):
    executable = 'vint'
    arguments = '{filename}'
    output_regex = re.compile(
        r'(?P<filename>.+):(?P<line>\d+):'
        r'(?P<column>\d+): (?P<message>.+)')

    def run(self, filename, file):
        '''
        Checks vimscript code for possible problems using ``vint-linter``.
        '''
        return self.lint(filename)



import re

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY
from coalib.settings.Setting import typed_list


class BootLintBear(LocalBear, Lint):
    executable = 'bootlint'
    output_regex = re.compile(
        r'(.+?):*(?P<line>\d*):+'
        r'(?P<col>\d*)\s(?P<severity>.)\d+'
        r'\s(?P<message>.+)')
    severity_map = {
        "W": RESULT_SEVERITY.NORMAL,
        "E": RESULT_SEVERITY.MAJOR
    }

    def _get_groupdict(self, match):
        groups = Lint._get_groupdict(self, match)
        if groups["line"] == "" and groups["col"] == "":
            del groups["line"], groups["col"]
        return groups

    def run(self, filename, file, bootlint_ignore: typed_list(str)=[]):
        '''
        Checks the code with ``bootlint`` on each file separately.

        :param bootlint_ignore: List of checkers to ignore.
        '''
        ignore = ','.join(part.strip() for part in bootlint_ignore)
        self.arguments = '--disable=' + ignore + " {filename}"
        return self.lint(filename)



import os

from bears.js.ESLintBear import ESLintBear
from tests.LocalBearTestHelper import verify_local_bear


test_good = """function addOne(i) {
    if (!isNaN(i)) {
        return i+1;
    }
    return i;
}

addOne(3);
""".splitlines(True)

test_bad = """function addOne(i) {
    if (i != NaN) {
        return i ++
    }
    else {
        return
    }
};
""".splitlines(True)

eslintconfig = os.path.join(os.path.dirname(__file__),
                            "test_files",
                            "eslintconfig.json")

ESLintBearTestWithConfig = verify_local_bear(ESLintBear,
                                             valid_files=(test_good,),
                                             invalid_files=(test_bad,),
                                             settings={"eslint_config":
                                                       eslintconfig})

ESLintBearWithoutConfig = verify_local_bear(ESLintBear,
                                            valid_files=(test_good, test_bad),
                                            invalid_files=(),)



import re

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY


class PHPLintBear(LocalBear, Lint):
    executable = 'php'
    arguments = '-l -n -d display_errors=On -d log_errors=Off {filename}'
    output_regex = re.compile(
        r'(?P<severity>\S+) error: '
        r'(?P<message>.*) in (?P<file_name>.*) on line (?P<line>\d+)')
    severity_map = {
        "Parse": RESULT_SEVERITY.MAJOR,
        "Fatal": RESULT_SEVERITY.MAJOR}

    def run(self, filename, file):
        '''
        Checks the code with `php -l`. This runs it on each file separately.
        '''
        return self.lint(filename)



import requests
import unittest
from queue import Queue
import requests_mock

from bears.general.InvalidLinkBear import InvalidLinkBear
from coalib.settings.Section import Section


def custom_matcher(request):
    change_url = False
    try:
        status_code = int(request.path_url[-3:])
        if status_code in range(300, 400):
            change_url = True
            url = "some_url"
        resp = requests.Response()
        if change_url:
            resp.url = url
        resp.status_code = status_code
        return resp
    except ValueError:
        raise requests.exceptions.RequestException("")


class InvalidLinkBearTest(unittest.TestCase):

    def setUp(self):
        self.section = Section("")

    def assertResult(self, valid_file=None, invalid_file=None):
        with requests_mock.Mocker() as m:
            InvalidLinkBear.check_prerequisites = lambda *args: True
            uut = InvalidLinkBear(self.section, Queue())
            m.add_matcher(custom_matcher)
            if valid_file:
                out = uut.execute("valid", valid_file)
                self.assertEqual(out, [])
            if invalid_file:
                out = uut.execute("invalid", invalid_file)
                self.assertNotEqual(out, [])
                self.assertNotEqual(out, None)

    def test_run(self):
        # Link Redirect
        valid_file = """
        http://httpbin.org/status/200
        """.splitlines()

        invalid_file = """
        http://httpbin.org/status/301
        http://coala.rtfd.org/302
        """.splitlines()

        self.assertResult(valid_file=valid_file, invalid_file=invalid_file)

        # Invalid Link Not Found
        valid_file = """
        http://httpbin.org/status/202
        """.splitlines()
        invalid_file = """
        http://httpbin.org/status/404
        http://httpbin.org/status/401
        """.splitlines()
        self.assertResult(valid_file=valid_file, invalid_file=invalid_file)

        # Invalid Link ServerError
        valid_file = """
        http://httpbin.org/status/202
        """.splitlines()
        invalid_file = """
        http://httpbin.org/status/500
        http://httpbin.org/status/503
        """.splitlines()
        self.assertResult(valid_file=valid_file, invalid_file=invalid_file)

        # Link Does Not Exist
        valid_file = """
           http://coala-analyzer.org/200\n
           http://coala-analyzer.org/200
           http://not a link dot com
           <http://lwn.net/200>
           'https://www.gnome.org/200'
           http://coala-analyzer.org/200.....
        """.splitlines()
        invalid_file = """
           http://coalaisthebest.com/
        """.splitlines()

        self.assertResult(valid_file=valid_file, invalid_file=invalid_file)

        # Mark down Links
        valid_file = """
            [coala](http://coala-analyzer.org/200)
            https://en.wikipedia.org/wiki/Hello_(Adele_song)/200
        """.splitlines()
        invalid_file = """
            http://coalaisthebest.com/
        """.splitlines()
        self.assertResult(valid_file=valid_file)

        # SphinxLinks
        valid_file = """
        |https://github.com/coala-analyzer/coala-bears/200|
        """.splitlines()
        self.assertResult(valid_file=valid_file)

    def test_check_prerequisites(self):
        with requests_mock.Mocker() as m:
            m.add_matcher(custom_matcher)
            self.assertIn("You are not", InvalidLinkBear.check_prerequisites())
            m.head("http://www.google.com")
            self.assertEqual(InvalidLinkBear.check_prerequisites(), True)



import re

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear


class ProseLintBear(LocalBear, Lint):
    executable = 'proselint'
    output_regex = re.compile(
        r'.+?:(?P<line>\d+):(?P<column>\d+): (?P<code>\S*) (?P<message>.+)')
    arguments = "{filename}"

    def run(self, filename, file):
        '''
        Checks the markdown file with Alex - Catch insensitive,
        inconsiderate writing.
        '''
        return self.lint(filename)



from bears.vimscript.VintBear import VintBear
from tests.LocalBearTestHelper import verify_local_bear


good_file = """
:let foo = 'bar'
:echo foo
""".splitlines(keepends=True)


bad_file = """
:let foo = "bar"
:echo foo
""".splitlines(keepends=True)


VintBearTest = verify_local_bear(VintBear,
                                 valid_files=(good_file,),
                                 invalid_files=(bad_file,))



import re
import json

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.misc.Shell import escape_path_argument
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY


def bool_or_str(value):
    try:
        return bool(value)
    except:
        return str(value)


def bool_or_int(value):
    try:
        return bool(value)
    except:
        return int(value)


class JSHintBear(LocalBear, Lint):
    executable = 'jshint'
    output_regex = re.compile(
        r'.+?: line (?P<line>\d+), col (?P<col>\d+), '
        r'(?P<message>.+) \((?P<severity>\S)\d+\)')
    severity_map = {
        "E": RESULT_SEVERITY.MAJOR,
        "W": RESULT_SEVERITY.NORMAL,
        "I": RESULT_SEVERITY.NORMAL}

    def config_file(self):
        if self.config_options is None:
            return None
        return self.config_options.splitlines()

    def run(self,
            filename,
            file,
            prohibit_bitwise: bool=True,
            prohibit_prototype_overwrite: bool=True,
            force_braces: bool=True,
            prohibit_type_coercion: bool=True,
            future_hostile: bool=False,
            prohibit_typeof: bool=False,
            es3: bool=False,
            force_filter_forin: bool=True,
            allow_funcscope: bool=False,
            iterator: bool=False,
            prohibit_arg: bool=True,
            prohibit_comma: bool=False,
            prohibit_non_breaking_whitespace: bool=True,
            prohibit_new: bool=False,
            prohibit_undefined: bool=True,
            prohibit_groups: bool=False,
            prohibit_variable_statements: bool=False,
            allow_missing_semicol: bool=False,
            allow_debugger: bool=False,
            allow_assignment_comparisions: bool=False,
            allow_eval: bool=False,
            allow_global_strict: bool=False,
            allow_increment: bool=False,
            allow_proto: bool=False,
            allow_scripturls: bool=False,
            allow_singleton: bool=False,
            allow_this_stmt: bool=False,
            allow_with_stmt: bool=False,
            using_mozilla: bool=False,
            allow_noyield: bool=False,
            allow_eqnull: bool=False,
            allow_last_semicolon: bool=False,
            allow_func_in_loop: bool=False,
            allow_expr_in_assignments: bool=False,
            use_es6_syntax: bool=False,
            use_es3_array: bool=False,
            environment_mootools: bool=False,
            environment_couch: bool=False,
            environment_jasmine: bool=False,
            environment_jquery: bool=False,
            environment_node: bool=False,
            environment_qunit: bool=False,
            environment_rhino: bool=False,
            environment_shelljs: bool=False,
            environment_prototypejs: bool=False,
            environment_yui: bool=False,
            environment_mocha: bool=True,
            environment_module: bool=False,
            environment_wsh: bool=False,
            environment_worker: bool=False,
            environment_nonstandard: bool=False,
            environment_browser: bool=True,
            environment_browserify: bool=False,
            environment_devel: bool=True,
            environment_dojo: bool=False,
            environment_typed: bool=False,
            environment_phantom: bool=False,
            maxerr: int=50,
            maxstatements: bool_or_int=False,
            maxdepth: bool_or_int=False,
            maxparams: bool_or_int=False,
            maxcomplexity: bool_or_int=False,
            shadow: bool_or_str=False,
            prohibit_unused: bool_or_str=True,
            allow_latedef: bool_or_str=False,
            es_version: int=5,
            jshint_config: str=""):
        '''
        Checks the code with jshint. This will run jshint over each file
        separately.

        :param prohibit_bitwise:                This option prohibits the use of
                                                bitwise operators.
        :param prohibit_prototype_overwrite:    This options prohibits
                                                overwritingprototypes of native
                                                objects such as ``Array``.
        :param force_braces:                    This option requires you to
                                                always put curly braces around
                                                blocks in loops and
                                                conditionals.
        :param prohibit_type_coercion:          This options prohibits the use
                                                of ``==`` and ``!=`` in favor
                                                of ``===`` and ``!==``.
        :param future_hostile:                  This option enables warnings
                                                about the use of identifiers
                                                which are defined in future
                                                versions of JavaScript.
        :param prohibit_typeof:                 This option suppresses warnings
                                                about invalid ``typeof``
                                                operator values.
        :param es3:                             This option tells JSHint that
                                                your code needs to adhere to
                                                ECMAScript 3 specification.
        :param force_filter_forin:              This option requires all
                                                ``for in`` loops to filter
                                                object's items.
        :param iterator:                        This option suppresses warnings
                                                about the ``__iterator__``
                                                property.
        :param allow_funcscope:                 This option suppresses warnings
                                                about declaring variables inside
                                                of control structures while
                                                accessing them later from
                                                outside.
        :param prohibit_arg:                    This option prohibits the use of
                                                ``arguments.caller`` and
                                                ``arguments.callee``.
        :param prohibit_comma:                  This option prohibits the use of
                                                the comma operator.
        :param prohibit_non_breaking_whitespace: This option warns about "non-
                                                breaking whitespace characters".
        :param prohibit_new:                    This option prohibits the use of
                                                constructor functions for
                                                side-effects.
        :param prohibit_undefined:              This option prohibits the use of
                                                explicitly undeclared variables.
        :param prohibit_groups:                 This option prohibits the use of
                                                the grouping operator when it is
                                                not strictly required.
        :param prohibit_variable_statements:    This option forbids the use of
                                                VariableStatements.
        :param allow_missing_semicol:           This option suppresses warnings
                                                about missing semicolons.
        :param allow_debugger:                  This option suppresses warnings
                                                about the ``debugger``
                                                statements.
        :param allow_assignment_comparisions:    This option suppresses warnings
                                                about the use of assignments in
                                                cases where comparisons are
                                                expected.
        :param allow_eval:                      This options suppresses warnings
                                                about the use of ``eval``
                                                function.
        :param allow_global_strict:             This option suppresses warnings
                                                about the use of global strict
                                                mode.
        :param allow_increment:                 This option suppresses warnings
                                                about the use of unary increment
                                                and decrement operators.
        :param allow_proto:                     This option suppresses warnings
                                                about the ``__proto__``
                                                property.
        :param allow_scripturls:                This option suppresses warnings
                                                about the use of script-targeted
                                                URLs.
        :param allow_singleton:                 This option suppresses warnings
                                                about constructions like
                                                ``new function () { ... }`` and
                                                ``new Object;`` sometimes used
                                                to produce singletons.
        :param allow_this_stmt:                 This option suppresses warnings
                                                about possible strict violations
                                                when the code is running in
                                                strict mode and ``this`` is used
                                                in a non-constructor function.
        :param allow_with_stmt:                 This option suppresses warnings
                                                about the use of the ``with``
                                                statement.
        :param using_mozilla:                   This options tells JSHint that
                                                your code uses Mozilla
                                                JavaScript extensions.
        :param allow_noyield:                   This option suppresses warnings
                                                about generator functions with
                                                no ``yield`` statement in them.
        :param allow_eqnull:                    This option suppresses warnings
                                                about ``== null`` comparisons.
        :param allow_last_semicolon:            This option suppresses warnings
                                                about missing semicolons for the
                                                last statement.
        :param allow_func_in_loop:              This option suppresses warnings
                                                about functions inside of loops.
        :param allow_expr_in_assignments:       This option suppresses warnings
                                                about the use of expressions
                                                where normally assignments or
                                                function calls are expected.
        :param use_es3_array:                   This option tells JSHintBear
                                                ES3 array elision elements,
                                                or empty elements are used.
        :param use_es3_array:                   This option tells JSHint
                                                ECMAScript 6 specific syntax is
                                                used.
        :param environment_mootools:            This option defines globals
                                                exposed by the Mootools.
        :param environment_couch:               This option defines globals
                                                exposed by CouchDB.
        :param environment_jasmine:             This option defines globals
                                                exposed by Jasmine.
        :param environment_jquery:              This option defines globals
                                                exposed by Jquery.
        :param environment_node:                This option defines globals
                                                exposed by Node.
        :param environment_qunit:               This option defines globals
                                                exposed by Qunit.
        :param environment_rhino:               This option defines globals
                                                exposed when the code is running
                                                inside rhino runtime
                                                environment.
        :param environment_shelljs:             This option defines globals
                                                exposed by the ShellJS.
        :param environment_prototypejs:         This option defines globals
                                                exposed by the Prototype.
        :param environment_yui:                 This option defines globals
                                                exposed by the YUI JavaScript
                                                Framework.
        :param environment_mocha:               This option defines globals
                                                exposed by the "BDD" and "TDD"
                                                UIs of the Mocha unit testing
                                                framework.
        :param environment_module:              This option informs JSHintBear
                                                that the input code describes an
                                                ECMAScript 6 module.
        :param environment_wsh:                 This option defines globals
                                                available when the code is
                                                running as a script for the
                                                Windows Script Host.
        :param environment_worker:              This option defines globals
                                                available when the code is
                                                running inside of a Web Worker.
        :param environment_nonstandard:         This option defines non-standard
                                                but widely adopted globals such
                                                as ``escape`` and ``unescape``.
        :param environment_browser:             This option defines globals
                                                exposed by modern browsers.
        :param environment_browserify:          This option defines globals
                                                available when using the
                                                Browserify.
        :param environment_devel:               This option defines globals that
                                                are usually used for debugging:
                                                ``console``, ``alert``, etc.
        :param environment_dojo:                This option defines globals
                                                exposed by the Dojo Toolkit.
        :param environment_typed:               This option defines globals for
                                                typed array constructors.
        :param environment_phantom:             This option defines globals
                                                available when your core is
                                                running inside of the PhantomJS
                                                runtime environment.
        :param maxerr:                          This options allows you to set
                                                the maximum amount of warnings
                                                JSHintBear will produce before
                                                giving up. Default is 50.
        :param maxstatements:                   Maximum number of statements
                                                allowed per function.
        :param maxdepth:                        This option lets you control how
                                                nested do you want your blocks
                                                to be.
        :param maxparams:                       Maximum number of formal
                                                parameters allowed per function.
        :param maxcomplexity:                   Maximum cyclomatic complexity in
                                                the code.
        :param shadow:                          This option suppresses warnings
                                                about variable shadowing i.e.
                                                declaring a variable that had
                                                been already declared somewhere
                                                in the outer scope.
                                                 - "inner" - check for variables
                                                             defined in the same
                                                             scope only
                                                 - "outer" - check for variables
                                                             defined in outer
                                                             scopes as well
                                                 - False   - same as inner
                                                 - True    - allow variable
                                                             shadowing
        :param prohibit_unused:                 This option generates warnings
                                                when variables are defined but
                                                never used. This can be set to
                                                ""vars"" to only check for
                                                variables, not function
                                                parameters,or ""strict"" to
                                                check all variables and
                                                parameters.
        :param allow_latedef:                   This option prohibits the use of
                                                a variable before it was
                                                defined.
                                                Setting this option to "nofunc"
                                                will allow function declarations
                                                to be ignored.
        :param es_version:                      This option is used to specify
                                                the ECMAScript version to which
                                                the code must adhere to.
        :param jshint_config:                   The location of the jshintrc
                                                config file. If this option is
                                                present all the above options
                                                are not used. Instead the
                                                .jshintrc file is used as the
                                                configuration file.
        '''
        self.arguments = '--verbose {filename}'

        if jshint_config:
            self.config_options = None
            self.arguments += (" --config "
                               + escape_path_argument(jshint_config))
        else:
            options = {
                "bitwise": prohibit_bitwise,
                "freeze": prohibit_prototype_overwrite,
                "curly": force_braces,
                "eqeqeq": prohibit_type_coercion,
                "futurehostile": future_hostile,
                "notypeof": prohibit_typeof,
                "es3": es3,
                "forin": force_filter_forin,
                "funcscope": allow_funcscope,
                "iterator": iterator,
                "noarg": prohibit_arg,
                "nocomma": prohibit_comma,
                "nonbsp": prohibit_non_breaking_whitespace,
                "nonew": prohibit_new,
                "undef": prohibit_undefined,
                "singleGroups": prohibit_groups,
                "varstmt": prohibit_variable_statements,
                "asi": allow_missing_semicol,
                "debug": allow_debugger,
                "boss": allow_assignment_comparisions,
                "evil": allow_eval,
                "globalstrict": allow_global_strict,
                "plusplus": allow_increment,
                "proto": allow_proto,
                "scripturl": allow_scripturls,
                "supernew": allow_singleton,
                "validthis": allow_this_stmt,
                "withstmt": allow_with_stmt,
                "moz": using_mozilla,
                "noyield": allow_noyield,
                "eqnull": allow_eqnull,
                "lastsemic": allow_last_semicolon,
                "loopfunc": allow_func_in_loop,
                "expr": allow_expr_in_assignments,
                "esnext": use_es6_syntax,
                "elision": use_es3_array,
                "mootools": environment_mootools,
                "couch": environment_couch,
                "jasmine": environment_jasmine,
                "jquery": environment_jquery,
                "node": environment_node,
                "qunit": environment_qunit,
                "rhino": environment_rhino,
                "shelljs": environment_shelljs,
                "prototypejs": environment_prototypejs,
                "yui": environment_yui,
                "mocha": environment_mocha,
                "module": environment_module,
                "wsh": environment_wsh,
                "worker": environment_worker,
                "nonstandard": environment_nonstandard,
                "browser": environment_browser,
                "browserify": environment_browserify,
                "devel": environment_devel,
                "dojo": environment_dojo,
                "typed": environment_typed,
                "phantom": environment_phantom,
                "maxerr": maxerr,
                "maxcomplexity": maxcomplexity,
                "maxdepth": maxdepth,
                "maxparams": maxparams,
                "maxstatements": maxstatements,
                "shadow": shadow,
                "unused": prohibit_unused,
                "latedef": allow_latedef,
                "esversion": es_version}

            self.config_options = json.dumps(options)
            self.arguments += " --config {config_file}"

        return self.lint(filename)



from bears.shell.ShellCheckBear import ShellCheckBear
from tests.LocalBearTestHelper import verify_local_bear


valid_file = """
#!/usr/bin/env sh
echo "Path is $PATH"
echo "$P"
""".splitlines(keepends=True)

invalid_file = """
#!/usr/bin/env sh
echo 'Path is $PATH'
echo $P
X = 24
X+=2    # += operator is supported by bash and not by sh
echo $X
""".splitlines(keepends=True)


ShellCheckBearTest = verify_local_bear(ShellCheckBear,
                                       valid_files=(valid_file,),
                                       invalid_files=(invalid_file,))



import os
import re
from queue import Queue
from shutil import which
from unittest.case import skipIf

from bears.python.PyLintBear import PyLintBear
from tests.LocalBearTestHelper import LocalBearTestHelper
from coalib.settings.Section import Section
from coalib.settings.Setting import Setting


@skipIf(which('pylint') is None, 'PyLint is not installed')
class PyLintBearTest(LocalBearTestHelper):

    def setUp(self):
        self.section = Section("test section")
        self.uut = PyLintBear(self.section, Queue())
        self.test_file = os.path.join(os.path.dirname(__file__),
                                      "test_files",
                                      "pylint_test.py")
        self.rc_file = os.path.join(os.path.dirname(__file__),
                                    "test_files",
                                    "pylint_config")

    def test_run(self):
        self.section.append(Setting("pylint_disable", ""))
        self.check_validity(
            self.uut,
            [],  # Doesn't matter, pylint will parse the file
            self.test_file,
            valid=False)

        # This is a special case because there's only one result yielded.
        # This was a bug once where the last result got ignored.
        self.section.append(Setting("pylint_disable", "E0211,W0611,C0111"))
        self.check_validity(self.uut, [], self.test_file, valid=False)

        self.section.append(
            Setting("pylint_disable", "E0211,W0611,C0111,W0311"))
        self.check_validity(self.uut, [], self.test_file)

        self.section.append(Setting("pylint_disable", "all"))
        self.check_validity(self.uut, [], self.test_file)

        self.section.append(Setting("pylint_enable", "C0111"))
        self.check_validity(self.uut, [], self.test_file, valid=False)

        self.section.append(Setting("pylint_cli_options", "--disable=all"))
        self.check_validity(self.uut, [], self.test_file)

    def test_rcfile(self):
        self.section.append(Setting("pylint_rcfile", re.escape(self.rc_file)))
        self.check_validity(self.uut, [], self.test_file)



from bears.ruby.RubySyntaxBear import RubySyntaxBear
from tests.LocalBearTestHelper import verify_local_bear

good_file = """
class HelloWorld
    def initialize(name)
        @name = name.capitalize
    end
    def sayHi
        puts "Hello #{@name}!"
    end
end
""".splitlines(keepends=True)


bad_file = """
class HelloWorld
    def initialize(name)
        @name = name.capitalize
    end
    def sayHi
        x = 1 # unused variables invoke a warning
        puts "Hello #{@name}!"
    end
""".splitlines(keepends=True)


RubySyntaxBearTest = verify_local_bear(RubySyntaxBear,
                                       valid_files=(good_file,),
                                       invalid_files=(bad_file,))



from bears.r.FormatRBear import FormatRBear
from tests.LocalBearTestHelper import verify_local_bear

good_file = """
1 + 1
if (TRUE) {
    x = 1  # inline comments
} else {
    x = 2
    print("Oh no... ask the right bracket to go away!")
}""".splitlines(keepends=True)


bad_file = """1+1
if(TRUE){
x=1  # inline comments
}else{
x=2;print('Oh no... ask the right bracket to go away!')}
""".splitlines(keepends=True)


FormatRBearTest = verify_local_bear(FormatRBear,
                                    valid_files=(good_file,),
                                    invalid_files=(bad_file,))



from bears.latex.LatexLintBear import LatexLintBear
from tests.LocalBearTestHelper import verify_local_bear


good_file = """
{.}
{ sometext }\\
""".splitlines(keepends=True)


bad_file = """
{ .}
{ Sometext \\
""".splitlines(keepends=True)


LatexLintBearTest = verify_local_bear(LatexLintBear,
                                      valid_files=(good_file,),
                                      invalid_files=(bad_file,))



from bears.go.GofmtBear import GofmtBear
from tests.LocalBearTestHelper import verify_local_bear

GofmtBear = verify_local_bear(
    GofmtBear,
    (['package main', '', 'func main() {', '\treturn 1', '}'],),
    (['package main', 'func main() {', '    return 1', '}'],))



from queue import Queue

from bears.python.PEP8Bear import PEP8Bear
from tests.LocalBearTestHelper import LocalBearTestHelper
from coalib.settings.Section import Section
from coalib.settings.Setting import Setting


class PEP8BearTest(LocalBearTestHelper):

    def setUp(self):
        self.section = Section('name')
        self.section.append(Setting('max_line_length', '80'))
        self.uut = PEP8Bear(self.section, Queue())

    def test_valid(self):
        self.check_validity(self.uut, ["import sys"])
        self.check_validity(self.uut, ["a = 1 + 1"])

    def test_line_length(self):
        self.check_validity(self.uut, ["a = 1 + 1 + 1 + 1 + 1 + 1 + 1"])
        self.section.append(Setting('max_line_length', '10'))
        self.check_validity(self.uut,
                            ["a = 1 + 1 + 1 + 1 + 1 + 1 + 1"],
                            valid=False)

    def test_indent_level(self):
        test_code = ['def func():\n',
                     '    pass\n']
        self.check_validity(self.uut, test_code)

        self.section.append(Setting('tab_width', '2'))
        self.check_validity(self.uut, test_code, valid=False)
        self.check_validity(self.uut, ['def func():\n', '  pass\n'])

    def test_disable_warnings(self):
        test_code = ['def func():\n',
                     '    pass\n',
                     'def func2():\n',
                     '    pass\n']
        self.check_validity(self.uut, test_code, valid=False)

        self.section.append(Setting('pep_ignore', 'E302'))
        self.check_validity(self.uut, test_code)

    def test_invalid(self):
        self.check_validity(self.uut, [""], valid=False)
        self.check_validity(self.uut, ["a=1+1"], valid=False)



import re

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.settings.Setting import typed_list


class CPPLintBear(LocalBear, Lint):
    executable = 'cpplint'
    output_regex = re.compile(
        r'(?P<filename>.+\..+):(?P<line>\d+):\s(?P<message>.+)')
    use_stderr = True

    def run(self,
            filename,
            file,
            max_line_length: int=80,
            cpplint_ignore: typed_list(str)=[],
            cpplint_include: typed_list(str)=[]):
        '''
        Checks the code with `cpplint` on each file separately.

        :param max_line_length: Maximum number of characters for a line.
        :param cpplint_ignore:  List of checkers to ignore.
        :param cpplint_include: List of checkers to explicitly enable.
        '''
        ignore = ','.join('-'+part.strip() for part in cpplint_ignore)
        include = ','.join('+'+part.strip() for part in cpplint_include)
        self.arguments = '--filter=' + ignore + ',' + include
        self.arguments += ' --linelength=' + str(max_line_length)
        self.arguments += ' {filename}'
        return self.lint(filename)



from queue import Queue

from bears.python.PyUnusedCodeBear import PyUnusedCodeBear
from tests.LocalBearTestHelper import LocalBearTestHelper
from coalib.settings.Section import Section


class PyUnusedCodeBearTest(LocalBearTestHelper):

    def setUp(self):
        self.uut = PyUnusedCodeBear(Section('name'), Queue())

    def test_valid(self):
        self.check_validity(self.uut, ["import sys; sys.do()"])
        self.check_validity(self.uut, ["a = 2; print(a)"])

    def test_invalid(self):
        self.check_validity(self.uut, ["import os"], valid=False)
        self.check_validity(self.uut, ["pass"], valid=False)



from bears.html.BootLintBear import BootLintBear
from tests.LocalBearTestHelper import verify_local_bear


good_file = """
<!DOCTYPE html>
<html lang="en">
    <head>
        <meta charset="utf-8" />
        <meta http-equiv="X-UA-Compatible" content="IE=edge" />
        <meta name="viewport" content="width=device-width, initial-scale=1" />
        <title>Test</title>
        <!--[if lt IE 9]>
            <script src="https://oss.maxcdn.com/html5shiv/3.7.2/
            html5shiv.min.js">
            </script>
            <script src="https://oss.maxcdn.com/respond/1.4.2/
            respond.min.js">
            </script>
        <![endif]-->
        <script src="../../lib/jquery.min.js"></script>

        <link rel="stylesheet" href="../../lib/qunit.css">
        <script src="../../lib/qunit.js"></script>
        <script src="../../../dist/browser/bootlint.js"></script>
        <script src="../generic-qunit.js"></script>
    </head>
    <body>

        <button type="submit">Submit</button>
        <button type="reset">Reset</button>
        <button type="button">Button</button>

        <div id="qunit"></div>
        <ol id="bootlint"></ol>
    </body>
</html>
""".splitlines(keepends=True)

bad_file = """
<html lang="en">
    <head>
        <meta http-equiv="X-UA-Compatible" content="IE=edge" />
        <meta name="viewport" content="width=device-width, initial-scale=1" />
        <title>Test</title>
        <!--[if lt IE 9]>
            <script src="https://oss.maxcdn.com/html5shiv/3.7.2/
            html5shiv.min.js"></script>
            <script src="https://oss.maxcdn.com/respond/1.4.2/
            respond.min.js"></script>
        <![endif]-->
        <script src="../../lib/jquery.min.js"></script>

        <link rel="stylesheet" href="../../lib/qunit.css">
        <script src="../../lib/qunit.js"></script>
        <script src="../../../dist/browser/bootlint.js"></script>
        <script src="../generic-qunit.js"></script>
    </head>
    <body>

        <button>No type set</button>
        <div>
          <div class="panel-body">
            <p>Something</p>
          </div>
        </div>

        <div id="qunit"></div>
        <ol id="bootlint">
          <li data-lint="Found one or more `<button>`s
           missing a `type` attribute."></li>
        </ol>
    </body>
</html>
""".splitlines(keepends=True)
# There's a missing type in <button> tag, missing DOCTYPE
# and panel has no body.

BootLintBearTest = verify_local_bear(BootLintBear,
                                     valid_files=(good_file,),
                                     invalid_files=(bad_file,))

BootLintBearDisableTest = verify_local_bear(
    BootLintBear,
    valid_files=(good_file, bad_file),
    invalid_files=(),
    settings={'bootlint_ignore': 'W001,W007,E001,E023'})



import os
import unittest

from clang.cindex import CursorKind

from bears.c_languages.codeclone_detection.ClangCountVectorCreator import (
    ClangCountVectorCreator)
from tests.c_languages import skip_if_no_clang


def no_condition(stack):
    return True


def is_call_argument(stack):
    for elem, child_num in stack:
        if elem.kind == CursorKind.CALL_EXPR:
            return True

    return False


@skip_if_no_clang()
class ClangCountVectorCreatorTest(unittest.TestCase):
    functions = sorted(["main(int, char *)", "test()"])

    def setUp(self):
        self.testfile = os.path.abspath(os.path.join(
            os.path.dirname(__file__),
            "sample.c"))

    def test_empty_counting(self):
        expected_results = {
            (6, "test()"): {},
            (12, "main(int, char *)"): {
                # Variables
                "i": [],
                "asd": [],
                "t": [],
                "args": [],
                # Globals
                "g": [],
                # Functions
                "smile": [],
                "printf": [],
                # Constants
                "5": [],
                '"i is %d"': []}}

        self.uut = ClangCountVectorCreator()
        cv_dict = self.uut.get_vectors_for_file(self.testfile)

        self.check_cv_dict(cv_dict, expected_results)

    def check_cv_dict(self, actual, expected):
        self.assertEqual(len(actual), len(expected), str(actual))
        self.assertEqual(sorted(actual.keys()), sorted(expected.keys()))

        for function in actual:
            self.assertEqual(len(actual[function]), len(expected[function]))
            self.assertEqual(sorted(actual[function].keys()),
                             sorted(expected[function].keys()))
            for variable in actual[function]:
                self.assertEqual(actual[function][variable].count_vector,
                                 expected[function][variable])

    def test_counting(self):
        expected_results = {
            (6, "test()"): {},
            (12, "main(int, char *)"): {
                # Variables
                "i": [4, 1],
                "asd": [1, 0],
                "t": [4, 1],
                "args": [2, 0],
                # Globals
                "g": [3, 1],
                # Functions
                "smile": [1, 1],
                "printf": [1, 1],
                # Constants
                "5": [1, 0],
                '"i is %d"': [1, 1]}}

        self.uut = ClangCountVectorCreator([no_condition, is_call_argument])
        cv_dict = self.uut.get_vectors_for_file(self.testfile)

        self.check_cv_dict(cv_dict, expected_results)



import re

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY


class ShellCheckBear(LocalBear, Lint):
    executable = 'shellcheck'
    severity_map = {
        "error": RESULT_SEVERITY.MAJOR,
        "warning": RESULT_SEVERITY.NORMAL,
        "info": RESULT_SEVERITY.INFO}
    output_regex = re.compile(
        r'(?P<file>(.+)):(?P<line>\d+):(?P<column>\d+):.'
        r'(?P<severity>error|warning|info):.(?P<message>(.+))')

    def run(self, filename, file, shell: str='sh'):
        '''
        Checks the given code with ``shellcheck``

        :param shell:  Target shell being used. Default is ``sh``.
        '''
        self.arguments = '--f gcc -s {shell} {filename}'.format(
            shell=shell, filename=filename)
        return self.lint(filename=filename)



import re

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear


class GoVetBear(LocalBear, Lint):
    executable = 'go'
    arguments = 'vet {filename}'
    output_regex = re.compile(
        r'(?P<file_name>.+):(?P<line>\d+): (?P<message>.*)\n')
    use_stderr = True

    def run(self, filename, file):
        '''
        Checks the code using `go vet`.
        '''
        return self.lint(filename)



from bears.js.JSComplexityBear import JSComplexityBear
from tests.LocalBearTestHelper import verify_local_bear

complexity_12 = """(function () {
  var foo = 1 && 1 || 0;
  if (foo) {
    if (1) {
    } else {
      if (5*2/10) {
        if (0 || 1) {
          if (foo) {
          if (1) {
          } else {
            if (5*2/10) {
              if (0 || 1) {
                return;
              }
              return;
            }
          }
        }
          return;
        }
        return;
      }
    }
  }
})()
""".splitlines(True)

complexity_4 = """(function () {
  var foo = 1 && 1 || 0;
  if (0 || 1) {
    return;
  }
})()
""".splitlines(True)

JSComplexityBearTest = verify_local_bear(JSComplexityBear,
                                         valid_files=(complexity_4,),
                                         invalid_files=(complexity_12,),
                                         tempfile_kwargs={"suffix": ".js"})

JSComplexityBearThresholdTest = verify_local_bear(JSComplexityBear,
                                                  valid_files=(),
                                                  invalid_files=(complexity_4,
                                                                 complexity_12),
                                                  settings={"cc_threshold": 2},
                                                  tempfile_kwargs={"suffix":
                                                                   ".js"})

# No output for non-js files
JSComplexityBearInvalidFileTest = verify_local_bear(JSComplexityBear,
                                                    valid_files=(complexity_4,
                                                                 complexity_12),
                                                    invalid_files=())



import re

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY
from coalib.settings.Setting import typed_list


class HTMLLintBear(LocalBear, Lint):
    executable = 'html_lint.py'
    output_regex = re.compile(
        r'(?P<line>\d+):(?P<column>\d+):\s'
        r'(?P<severity>Error|Warning|Info):\s(?P<message>.+)'
    )
    severity_map = {
        "Info": RESULT_SEVERITY.INFO,
        "Warning": RESULT_SEVERITY.NORMAL,
        "Error": RESULT_SEVERITY.MAJOR
    }

    def run(self,
            filename,
            file,
            htmllint_ignore: typed_list(str)=[]):
        '''
        Checks the code with `html_lint.py` on each file separately.

        :param htmllint_include: List of checkers to ignore.
        '''
        ignore = ','.join(part.strip() for part in htmllint_ignore)
        self.arguments = '--disable=' + ignore
        self.arguments += " {filename}"
        return self.lint(filename)



from bears.python.PyImportSortBear import PyImportSortBear
from tests.LocalBearTestHelper import verify_local_bear

PyImportSortBearTest = verify_local_bear(PyImportSortBear,
                                         (["import os\n", "import sys\n"],
                                          ("import os\n", "import sys\n")),
                                         (["import sys\n", "import os\n"],
                                          ("import sys\n", "import os\n")))



from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear


class FormatRBear(Lint, LocalBear):
    executable = "Rscript"
    arguments = "-e 'library(formatR)' -e 'formatR::tidy_source({filename})'"
    prerequisite_command = ['Rscript', '-e', "'library(formatR)'"]
    prerequisite_fail_msg = "Please install formatR for this bear to work."
    diff_message = "Formatting can be improved."
    gives_corrected = True

    def process_output(self, output, filename, file):
        output = output[:-1] + (output[-1].strip()+"\n",)
        return Lint.process_output(self, output, filename, file)

    def run(self, filename, file):
        '''
        This bear checks and corrects formatting of R Code using
        known formatR utility.
        '''
        return self.lint(filename, file)



import re

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY


class InferBear(LocalBear, Lint):
    executable = 'infer'
    arguments = '-npb -- javac {filename}'
    output_regex = re.compile(
        r'(?P<file_name>.+):'
        r'(?P<line>.+): '
        r'(?P<severity>error|warning): '
        r'(?P<message>.*)')
    severity_map = {
        "error": RESULT_SEVERITY.MAJOR,
        "warning": RESULT_SEVERITY.NORMAL}

    def run(self, filename, file):
        '''
        Checks the code with ``infer``.
        '''
        return self.lint(filename)



import os
from queue import Queue
from shutil import which
from unittest.case import skipIf

from bears.go.GoVetBear import GoVetBear
from tests.LocalBearTestHelper import LocalBearTestHelper
from coalib.settings.Section import Section


@skipIf(which('go') is None, 'go is not installed')
class GoVetBearTest(LocalBearTestHelper):

    def setUp(self):
        self.section = Section("test section")
        self.uut = GoVetBear(self.section, Queue())
        self.good_file = os.path.join(os.path.dirname(__file__),
                                      "test_files",
                                      "vet_good.go")
        self.bad_file = os.path.join(os.path.dirname(__file__),
                                     "test_files",
                                     "vet_bad.go")

    def test_run(self):
        self.check_validity(self.uut, [], self.good_file)
        self.check_validity(self.uut, [], self.bad_file, valid=False)



from unittest.case import skip, skipIf

from clang.cindex import Index, LibclangError


def skip_if_no_clang():
    """
    Decorate your test with this to skip it if clang isn't present.
    """
    try:
        Index.create()
        return skipIf(False, '')
    except LibclangError as error:
        return skip(str(error))



from bears.general.KeywordBear import KeywordBear
from tests.LocalBearTestHelper import verify_local_bear

test_file = """
test line fix me
to do
error fixme
""".splitlines(keepends=True)


KeywordBearTest = verify_local_bear(KeywordBear,
                                    valid_files=(test_file,),
                                    invalid_files=(["test line FIXME"],
                                                   ["test line todo"],
                                                   ["test line warNING"],
                                                   ["test line ERROR"]),
                                    settings={
                                       "cs_keywords": "FIXME, ERROR",
                                       "ci_keywords": "todo, warning"})



from queue import Queue

from bears.general.LineCountBear import LineCountBear
from tests.LocalBearTestHelper import LocalBearTestHelper
from coalib.results.Result import RESULT_SEVERITY, Result
from coalib.settings.Section import Section


class LineCountBearTest(LocalBearTestHelper):

    def setUp(self):
        self.uut = LineCountBear(Section("name"), Queue())

    def test_run(self):
        self.check_results(
            self.uut,
            ["1", "2", "3"],
            [Result.from_values("LineCountBear",
                                "This file has {count} lines.".format(count=3),
                                severity=RESULT_SEVERITY.INFO,
                                file="default")],
            filename="default")
        self.check_results(
            self.uut,
            [],
            [Result.from_values("LineCountBear",
                                "This file has {count} lines.".format(count=0),
                                severity=RESULT_SEVERITY.INFO,
                                file="default")],
            filename="default")



from bears.js.JSHintBear import JSHintBear
from tests.LocalBearTestHelper import verify_local_bear
from coalib.misc.ContextManagers import prepare_file

test_file1 = """
var name = (function() { return 'Anton' }());
""".splitlines(keepends=True)


test_file2 = """
function () {
}()
""".splitlines(keepends=True)


test_file3 = """
var a = (function() {
  return 0;
}());
""".splitlines(keepends=True)


config_file = """
{
  "lastsemic": true,
  "maxlen": 80
}
""".splitlines(keepends=True)


settings = {
  "maxstatements": "False",
  "maxparams": 10,
  "prohibit_unused": "False",
  "shadow": "False",
  "allow_last_semicolon": "True",
  "es_version": 3,
  "allow_latedef": "no_func"}


JSHintBearTest = verify_local_bear(JSHintBear,
                                   valid_files=(),
                                   invalid_files=(test_file1, test_file2,
                                                  test_file3))


with prepare_file(config_file,
                  filename=None,
                  force_linebreaks=True,
                  create_tempfile=True) as (conf_lines, conf_file):
    JSHintBearConfigFileTest = verify_local_bear(
        JSHintBear,
        valid_files=(test_file1, test_file2),
        invalid_files=(),
        settings={"jshint_config": conf_file})


JSHintBearCoafileTest = verify_local_bear(
    JSHintBear,
    invalid_files=(),
    valid_files=(test_file3, ),
    settings=settings)



from bears.css.CSSLintBear import CSSLintBear
from tests.LocalBearTestHelper import verify_local_bear

good_file = """
.class {
  font-weight: 400;
  font-size: 5px;
}
""".splitlines(keepends=True)


bad_file = """
.class {
  font-weight: 400
  font-size: 5px;
}
""".splitlines(keepends=True)


CSSLintBearTest = verify_local_bear(CSSLintBear,
                                    valid_files=(good_file,),
                                    invalid_files=(bad_file,))



from clang.cindex import Index, TranslationUnit

from bears.c_languages.ClangBear import clang_available
from coalib.bears.GlobalBear import GlobalBear


class ClangASTPrintBear(GlobalBear):
    check_prerequisites = classmethod(clang_available)

    def print_node(self, cursor, filename, before="", spec_before=""):
        '''
        Prints this node and all child nodes recursively in the style of:

        ::

            -node
            |-child
            `-another child
            |- child of child
            `- last child of child

        :param cursor:      The node to print. (Clang cursor.)
        :param before:      What to print before the node.
        :param spec_before: What to print before this node but to replace with
                            spaces for child nodes.
        '''
        file = cursor.location.file

        if file is not None and file.name == filename:
            self.debug(
                before + spec_before + "-" + str(cursor.displayname),
                str(cursor.kind),
                "Lines",
                str(cursor.extent.start.line) + "-" +
                str(cursor.extent.end.line),
                "(" + " ".join(str(token.spelling)
                               for token in cursor.get_tokens()) + ")")

        children = list(cursor.get_children())

        if len(children) > 0:
            for child in children[:-1]:
                self.print_node(child,
                                filename,
                                before + len(spec_before)*" " + "|")

            self.print_node(children[-1],
                            filename,
                            before + len(spec_before)*" ",
                            "`")

    def run(self):
        """
        This bear is meant for debugging purposes relating to clang. It just
        prints out the whole AST for a file to the DEBUG channel.
        """
        for filename, file in sorted(self.file_dict.items()):
            root = Index.create().parse(
                filename,
                options=TranslationUnit.PARSE_DETAILED_PROCESSING_RECORD).cursor

            self.print_node(root, filename)



from bears.go.GoLintBear import GoLintBear
from tests.LocalBearTestHelper import verify_local_bear

good_file = """
// Test that blank imports in package main are not flagged.
// OK

// Binary foo ...
package main

import _ "fmt"

import (
  "os"
  _ "path"
)

var _ os.File // for "os"
""".splitlines(keepends=True)


bad_file = """
package pkg

func addOne(x int) int {
  x += 1
  return x
}

func subOneInLoop(y int) {
  for ; y > 0; y -= 1 {
  }
}
""".splitlines(keepends=True)


GoLintBearTest = verify_local_bear(GoLintBear,
                                   valid_files=(good_file,),
                                   invalid_files=(bad_file,))


GoLintBearWithSettingsTest = verify_local_bear(
    GoLintBear,
    valid_files=(bad_file, good_file),
    invalid_files=(),
    settings={"golint_cli_options": "-min_confidence=1"})



import re

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY


class LatexLintBear(LocalBear, Lint):
    executable = 'chktex'
    output_regex = re.compile(r'(?P<severity>Error|Warning) (?P<num>[0-9]+)'
                              r' in (?P<file_name>\S+) line (?P<line>[0-9]+)'
                              r': (?P<message>.*)')
    severity_map = {'Warning': RESULT_SEVERITY.NORMAL,
                    'Error': RESULT_SEVERITY.MAJOR}
    arguments = "{filename}"

    def run(self, filename, file):
        '''
        Checks the code with `chktex`.
        '''
        return self.lint(filename)



import os

from bears.scala.ScalaLintBear import ScalaLintBear
from tests.LocalBearTestHelper import verify_local_bear
from coalib.misc.ContextManagers import prepare_file

good_file = """
object HelloWorld {
  def main(args: Array[String]): Unit = {
    println("Hello, world!")
  }
}
""".splitlines(keepends=True)

bad_file = """
object HelloWorld {
  def main(args: Array[String]): Unit = {
    println("Hello, world!")
    var x = 10
  }
}
""".splitlines(keepends=True)

conf_file = os.path.join(os.path.dirname(__file__),
                         "test_files",
                         "scala_config.xml")


ScalaLintBearTest = verify_local_bear(
                        ScalaLintBear,
                        valid_files=(good_file,),
                        invalid_files=(bad_file,),
                        tempfile_kwargs={"suffix": ".scala"}
                        )


ScalaLintBearConfigTest = verify_local_bear(
                              ScalaLintBear,
                              valid_files=(good_file, bad_file),
                              invalid_files=(),
                              settings={"scalalint_config": conf_file},
                              tempfile_kwargs={"suffix": ".scala"}
                              )



from bears.haskell.HaskellLintBear import HaskellLintBear
from tests.LocalBearTestHelper import verify_local_bear

good_file = """
myconcat = (++)
""".splitlines(keepends=True)

bad_file = """
myconcat a b = ((++) a b)
""".splitlines(keepends=True)

HaskellLintBearTest = verify_local_bear(HaskellLintBear,
                                        valid_files=(good_file,),
                                        invalid_files=(bad_file,),
                                        tempfile_kwargs={"suffix": ".hs"})



import re

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear


class CPPCleanBear(LocalBear, Lint):
    executable = 'cppclean'
    output_regex = re.compile(
        r'(?P<file_name>[^,:]+):(?P<line>\d+):(?P<message>.*)')
    use_stdout = True
    arguments = '{filename}'

    def run(self, filename, file):
        '''
        Checks code with `cppclean`.
        '''
        return self.lint(filename)



from bears.natural_language.AlexBear import AlexBear
from tests.LocalBearTestHelper import verify_local_bear

good_file = ["Their network looks good."]

bad_file = ["His network looks good."]


AlexBearTest = verify_local_bear(AlexBear,
                                 valid_files=(good_file,),
                                 invalid_files=(bad_file,))



from bears.go.GoReturnsBear import GoReturnsBear
from tests.LocalBearTestHelper import verify_local_bear

good_file1 = """import "errors"

func F() (*MyType, int, error) { return nil, 0, errors.New("foo") }"""\
.splitlines(keepends=True)

good_file2 = """package main

import "os"

func main() {
\tf, _ := os.Open("foo")
}""".splitlines(keepends=True)

bad_file1 = """
func F() (*MyType, int, error) { return errors.New("foo") }
""".splitlines(keepends=True)

bad_file2 = """
package main

func main() {
    f, _ := os.Open("foo")
    return nil, 0, errors.New("foo")
}

""".splitlines(keepends=True)

a = ['import "errors"', '',
     'func F() (*MyType, int, error) { return nil, 0, errors.New("foo") }']
b = ['package main', '', 'import "os"', '',
     'func main() {', '\tf, _ := os.Open("foo")', '}']

GoReturnsBearTest = verify_local_bear(
    GoReturnsBear,
    (good_file1,
     good_file2,),
    (bad_file1, bad_file2,))



from bears.scss.SCSSLintBear import SCSSLintBear
from tests.LocalBearTestHelper import verify_local_bear


good_file = """
.btn-primary {
  &:hover {
    background-color: darken($btn-primary-bg, 3%);
  }
}
""".splitlines(keepends=True)

bad_file = """
.btn-primary {
  &:hover {
    background-color: darken($btn-primary-bg, 3%)
}
""".splitlines(keepends=True)


SCSSLintBearTest = verify_local_bear(SCSSLintBear,
                                     valid_files=(good_file,),
                                     invalid_files=(bad_file,))



from bears.python.RadonBear import RadonBear
from tests.LocalBearTestHelper import verify_local_bear

test_file1 = """
def simple():
    pass
""".splitlines(keepends=True)


test_file2 = """
class class1():
    pass
""".splitlines(keepends=True)


RadonBearNoReportsTest = verify_local_bear(RadonBear,
                                           valid_files=(test_file1, test_file2),
                                           invalid_files=(),
                                           settings={
                                               "radon_ranks_info": "",
                                               "radon_ranks_normal": "",
                                               "radon_ranks_major": ""})


RadonBearReportsTest = verify_local_bear(RadonBear,
                                         valid_files=(),
                                         invalid_files=(test_file1, test_file2),
                                         settings={
                                             "radon_ranks_info": "",
                                             "radon_ranks_normal": "A",
                                             "radon_ranks_major": ""})



from bears.dart.DartLintBear import DartLintBear
from tests.LocalBearTestHelper import verify_local_bear


good_file = """
printNumber(num aNumber) {
  print('The number is $aNumber.');
}

main() {
  var answer = 42;          // The meaning of life.
  printNumber(answer);
}
""".splitlines(keepends=True)


bad_file = """
printNumber(num aNumber) {
  print('The number is $aNumber.')
}

main() {
  var answer = 42;          // The meaning of life.
  printNumber(answer)
}
""".splitlines(keepends=True)


DartLintBearTest = verify_local_bear(DartLintBear,
                                     valid_files=(good_file,),
                                     invalid_files=(bad_file,),
                                     tempfile_kwargs={"suffix": ".dart"})



import unittest
from math import sqrt

from bears.c_languages.codeclone_detection.CountVector import CountVector


class CountVectorTest(unittest.TestCase):

    def test_simple_creation(self):
        uut = CountVector("varname")
        self.assertEqual(uut.count_vector, [])
        uut = CountVector("varname", [])
        self.assertEqual(uut.count_vector, [])
        self.assertRaises(AssertionError,
                          CountVector,
                          "varname",
                          conditions=[],
                          weightings=[2])

    def test_len(self):
        uut = CountVector("varname")
        self.assertEqual(len(uut), 0)

        uut = CountVector("varname", conditions=[lambda x: x])
        self.assertEqual(len(uut), 1)

    def test_counting(self):
        uut = CountVector("varname",
                          conditions=[lambda cursor, stack: cursor and stack])
        self.assertEqual(uut.count_vector, [0])
        uut.count_reference(True, True)
        self.assertEqual(uut.count_vector, [1])
        uut.count_reference(True, False)
        self.assertEqual(uut.count_vector, [1])
        self.assertEqual(uut.unweighted, [1])

    def test_weighting(self):
        uut = CountVector("varname",
                          conditions=[lambda cursor, stack: cursor and stack,
                                      lambda cursor, stack: cursor],
                          weightings=[2, 1])
        uut.count_reference(True, True)
        self.assertEqual(uut.count_vector, [2, 1])
        self.assertEqual(uut.unweighted, [1, 1])
        uut.count_reference(True, False)
        self.assertEqual(uut.count_vector, [2, 2])
        self.assertEqual(uut.unweighted, [1, 2])

    def test_conversions(self):
        uut = CountVector("varname",
                          conditions=[lambda cursor, stack: cursor and stack],
                          weightings=[2])
        uut.count_reference(True, True)
        self.assertEqual(str(uut), "[2]")
        self.assertEqual(list(uut), [2])

    def test_cloning(self):
        uut = CountVector("varname",
                          conditions=[lambda cursor, stack: cursor and stack],
                          weightings=[2])
        uut.count_reference(True, True)
        clone = uut.create_null_vector("test")
        self.assertEqual(clone.name, "test")
        self.assertEqual(clone.weightings, uut.weightings)
        self.assertEqual(clone.conditions, uut.conditions)
        self.assertEqual(clone.count_vector, [0])

    def test_abs(self):
        uut = CountVector("varname",
                          conditions=[lambda x: True, lambda x: x])
        self.assertEqual(abs(uut), 0)
        uut.count_reference(True)
        self.assertEqual(abs(uut), sqrt(2))
        uut.count_reference(False)
        self.assertEqual(abs(uut), sqrt(5))

    def check_difference(self,
                         cv1,
                         cv2,
                         expected_difference,
                         diff_function):
        """
        Checks the difference between the given count vectors through a
        given difference function.

        :param cv1:                 List of counts to put in the first CV.
        :param cv2:                 List of counts to put in the second CV.
        :param expected_difference: The expected difference value.
        :param diff_function:       The name of the member function to check
                                    (string).
        """
        # Create empty CountVector objects
        count_vector1 = CountVector("",
                                    conditions=[lambda: False for i in cv1])
        count_vector2 = CountVector("",
                                    conditions=[lambda: False for i in cv2])
        # Manually hack in the test values
        count_vector1.count_vector = cv1
        count_vector2.count_vector = cv2

        self.assertEqual(getattr(count_vector1, diff_function)(count_vector2),
                         expected_difference,
                         "Difference value for vectors {} and {} doesnt match"
                         ".".format(cv1, cv2))
        self.assertEqual(getattr(count_vector2, diff_function)(count_vector1),
                         expected_difference,
                         "The difference operation is not symmetric.")

    def test_difference(self):
        # For each tuple first two items are CVs to compare, third is dif value
        count_vector_difference_matrix = [
            ([], [], 0),
            ([0], [0], 0),
            ([1], [1], 0),
            ([100], [100], 0),

            ([0], [100], 100),
            ([0], [1], 1),
            ([0, 1], [1, 0], sqrt(2)),

            ([0, 1], [1, 1], 1),
            ([0, 2], [1, 2], 1),
            ([4], [3], 1),
            ([0, 4], [0, 3], 1)]  # Zeros don't matter

        for elem in count_vector_difference_matrix:
            self.check_difference(*elem, diff_function="difference")

    def test_maxabs(self):
        # For each tuple first two items are CVs to compare, third is dif value
        count_vector_difference_matrix = [
            ([], [], 0),
            ([0], [0], 0),
            ([1], [1], 1),
            ([100], [100], 100),

            ([0], [100], 100),
            ([0], [1], 1),
            ([0, 1], [1, 0], sqrt(2))]

        for elem in count_vector_difference_matrix:
            self.check_difference(*elem, diff_function="maxabs")



import os

# Path to the bears directory
bears_root = os.path.dirname(__file__)

VERSION_FILE = os.path.join(bears_root, "VERSION")
with open(VERSION_FILE, 'r') as ver:
    VERSION = ver.readline().strip()



from bears.js.JSONFormatBear import JSONFormatBear
from tests.LocalBearTestHelper import verify_local_bear

test_file1 = """{
    "a": 5,
    "b": 5
}""".splitlines(keepends=True)


test_file2 = """{
    "b": 5,
    "a": 5
}""".splitlines(keepends=True)

test_file3 = """{
   "b": 5,
   "a": 5
}""".splitlines(keepends=True)


unicode_file = """{
    "⌘": 5
}""".splitlines(keepends=True)


JSONFormatBearTest = verify_local_bear(JSONFormatBear,
                                       valid_files=(test_file1, test_file2),
                                       invalid_files=(test_file3,
                                                      unicode_file,
                                                      [""],
                                                      ["random stuff"],
                                                      ['{"a":5,"b":5}']))


JSONFormatBearSortTest = verify_local_bear(JSONFormatBear,
                                           valid_files=(test_file1,),
                                           invalid_files=(test_file2,),
                                           settings={"json_sort": "true"})


JSONFormatBearTabWidthTest = verify_local_bear(JSONFormatBear,
                                               valid_files=(test_file3,),
                                               invalid_files=(test_file2,),
                                               settings={"tab_width": "3"})


JSONFormatBearUnicodeTest = verify_local_bear(JSONFormatBear,
                                              valid_files=(unicode_file,),
                                              invalid_files=(),
                                              settings={'keep_unicode': 'true'})



from bears.r.RLintBear import RLintBear
from tests.LocalBearTestHelper import verify_local_bear

good_file = """
fun <- function(one){
  one_plus_one <- one + 1
  four <- matrix(1:10, nrow =2)
  print(one_plus_one, four)
}""".splitlines(keepends=True)


bad_file = """
fun = function(one)
{
    one.plus.one <- one + 1
    four <-  matrix(1:10,nrow =2)
    print(one_plus_one , four)
}
""".splitlines(keepends=True)


RLintBearTest = verify_local_bear(RLintBear,
                                  invalid_files=(bad_file,),
                                  valid_files=(good_file,),
                                  tempfile_kwargs={"suffix": ".R"})



from bears.c_languages.GNUIndentBear import GNUIndentBear
from tests.LocalBearTestHelper import verify_local_bear

test_file1 = """
int
main ()
{
    return 0;
}""".splitlines(keepends=True)

test_file2 = """
int
main ()
{
  return 0;
}""".splitlines(keepends=True)


test_file3 = """
int
main ()
{
\treturn 0;
}""".splitlines(keepends=True)


test_file4 = """
int main() {
  return 0;
}""".splitlines(keepends=True)


GNUIndentBearTest = verify_local_bear(GNUIndentBear,
                                      valid_files=(test_file1,),
                                      invalid_files=(test_file2,
                                                     test_file3,
                                                     test_file4),
                                      settings={"use_spaces": "true",
                                                "max_line_length": "80"})


GNUIndentBearWithTabTest = verify_local_bear(GNUIndentBear,
                                             valid_files=(test_file3,),
                                             invalid_files=(test_file1,
                                                            test_file2,
                                                            test_file4),
                                             settings={"use_spaces": "nope",
                                                       "max_line_length": "80"})


GNUIndentBearGNUIndentWidthTest = verify_local_bear(GNUIndentBear,
                                                    valid_files=(test_file2,),
                                                    invalid_files=(test_file1,
                                                                   test_file3,
                                                                   test_file4),
                                                    settings={"use_spaces":
                                                              "true",
                                                              "max_line_length":
                                                              "80",
                                                              "tab_width": "2"})



import unittest

from bears.c_languages.codeclone_detection.CloneDetectionRoutines import (
    relative_difference)


class CloneDetectionRoutinesTest(unittest.TestCase):

    def test_relative_difference(self):
        # This test is needed because our exclusion heuristic is grown so
        # good that during our "real code" tests some corner cases just
        # don't happen anymore. For simplicity we're whitebox-testing this
        # function, its whole purpose of life is to return 1 if it should
        # normalize with a zero (second parameter).

        self.assertEqual(relative_difference(0, 1), 0)
        self.assertEqual(relative_difference(1, 1), 1)
        self.assertEqual(relative_difference(0, 0), 1)
        self.assertEqual(relative_difference(1, 0), 1)
        self.assertEqual(relative_difference(0.5, 2), 0.25)



import json

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.results.Diff import Diff
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY
from coalib.misc.Shell import escape_path_argument
from coalib.results.Result import Result


class ESLintBear(LocalBear, Lint):
    executable = 'eslint'
    severity_map = {
        2: RESULT_SEVERITY.MAJOR,
        1: RESULT_SEVERITY.NORMAL,
        0: RESULT_SEVERITY.INFO
    }
    use_stdin = True
    gives_corrected = True

    def run(self, filename, file, eslint_config: str=""):
        '''
        Checks the code with eslint. This will run eslint over each of the files
        seperately.

        :param eslint_config: The location of the .eslintrc config file.
        '''
        self.arguments = '--no-ignore --no-color -f=json --stdin'
        if eslint_config:
            self.arguments += (" --config "
                               + escape_path_argument(eslint_config))

        return self.lint(filename, file)

    def _process_corrected(self, output, filename, file):
        output = json.loads("".join(output))
        lines = "".join(file)

        assert len(output) == 1

        for result in output[0]['messages']:
            if 'fix' not in result:
                diffs = None
            else:
                fix = result['fix']
                start, end = fix['range']
                replacement_text = fix['text']
                new_output = lines[:start] + replacement_text + lines[end:]
                diffs = {filename: Diff.from_string_arrays(
                    lines.splitlines(True), new_output.splitlines(True))}

            yield Result.from_values(
                origin="{class_name} ({rule})".format(
                    class_name=self.__class__.__name__, rule=result['ruleId']),
                message=result['message'],
                file=filename,
                diffs=diffs,
                severity=self.severity_map[result['severity']],
                line=result['line'])



from csv import DictReader
from io import StringIO

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.results.Result import Result
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY


def convert_if_not_empty(value: str, conversion):
    """
    Returns the value converted if it is not None or empty.

    :param value:      The value to convert.
    :param conversion: The conversion callable.
    :return:           None or the converted value.
    """
    if value is not None and value != '':
        return conversion(value)

    return None


class CoffeeLintBear(LocalBear, Lint):
    executable = 'coffeelint'
    arguments = '--reporter=csv {filename}'
    severity_map = {'warn': RESULT_SEVERITY.NORMAL,
                    'error': RESULT_SEVERITY.MAJOR}

    def run(self, filename, file):
        """
        Coffeelint's your files!
        """
        return self.lint(filename)

    def _process_issues(self, output, filename):
        reader = DictReader(StringIO("".join(output)))

        for row in reader:
            try:
                yield Result.from_values(
                    origin=self,
                    message=row['message'],
                    file=filename,
                    line=convert_if_not_empty(row['lineNumber'], int),
                    end_line=convert_if_not_empty(row['lineNumberEnd'], int),
                    severity=self.severity_map[row['level']])
            except KeyError:  # Invalid CSV line, ignore
                pass



import os
import unittest
from queue import Queue

from bears.c_languages.codeclone_detection.ClangCloneDetectionBear import (
    ClangCloneDetectionBear)
from bears.c_languages.codeclone_detection.ClangFunctionDifferenceBear import (
    ClangFunctionDifferenceBear)
from tests.BearTestHelper import generate_skip_decorator
from coalib.settings.Section import Section
from coalib.settings.Setting import Setting


@generate_skip_decorator(ClangCloneDetectionBear)
class ClangCloneDetectionBearTest(unittest.TestCase):

    def setUp(self):
        self.base_test_path = os.path.abspath(os.path.join(
            os.path.dirname(__file__),
            "clone_detection_samples"))
        self.section = Section("default")
        self.section.append(Setting("files", "", origin=self.base_test_path))
        self.section.append(Setting("max_clone_difference", "0.308"))
        self.clone_files = [os.listdir(os.path.join(self.base_test_path,
                                                    "clones"))]

    def test_dependencies(self):
        self.assertIn(ClangFunctionDifferenceBear,
                      ClangCloneDetectionBear.get_dependencies())

    def test_configuration(self):
        self.section.append(Setting("average_calculation", "true"))
        self.section.append(Setting("poly_postprocessing", "false"))
        self.section.append(Setting("exp_postprocessing", "true"))

        self.clone_files = [
            os.path.join(self.base_test_path, "clones", "s4c.c")]

        # Ignore the results, it may be possible that it still passes :)
        self.check_clone_detection_bear(self.clone_files,
                                        lambda results, msg: True)

    def test_non_clones(self):
        self.non_clone_files = [
            os.path.join(self.base_test_path, "non_clones", elem)
            for elem in os.listdir(os.path.join(self.base_test_path,
                                                "non_clones"))]

        self.check_clone_detection_bear(self.non_clone_files,
                                        lambda results, msg:
                                        self.assertEqual(results, [], msg))

    def test_clones(self):
        self.clone_files = [
            os.path.join(self.base_test_path, "clones", elem)
            for elem in os.listdir(os.path.join(self.base_test_path,
                                                "clones"))]

        self.check_clone_detection_bear(self.clone_files,
                                        lambda results, msg:
                                        self.assertNotEqual(results, [], msg))

    def check_clone_detection_bear(self, files, result_check_function):
        """
        Checks the results of the CloneDetectionBear with the given function.

        :param files:                 The files to check. Each will be checked
                                      on its own.
        :param result_check_function: A function yielding an exception if the
                                      results are invalid.
        """
        for file in files:
            difference_results = ClangFunctionDifferenceBear(
                {file: ""},
                self.section,
                Queue()).run_bear_from_section([], {})
            uut = ClangCloneDetectionBear(
                {file: ""},
                self.section,
                Queue())
            arg_dict = {"dependency_results":
                        {ClangFunctionDifferenceBear.__name__:
                         list(difference_results)}}

            result_check_function(
                list(uut.run_bear_from_section([], arg_dict)),
                "while analyzing "+file)



import os

from bears.cmake.CMakeLintBear import CMakeLintBear
from coalib.misc.ContextManagers import prepare_file
from tests.LocalBearTestHelper import verify_local_bear


good_file = ['project(FooBar C)\n', 'set(VERSION 0)\n']

bad_file_mixes_case = ['ProJeCt(FooBar C)\n', 'seT(VERSION 0)\n']


conf_file = os.path.join(os.path.dirname(__file__),
                         "test_files",
                         "cmake_config.txt")


CMakeLintBearTest = verify_local_bear(
    CMakeLintBear,
    valid_files=(good_file,),
    invalid_files=(bad_file_mixes_case,),
    tempfile_kwargs={"suffix": ".cmake"})


CMakeLintBearConfigTest = verify_local_bear(
    CMakeLintBear,
    valid_files=(good_file, bad_file_mixes_case),
    invalid_files=(),
    tempfile_kwargs={"suffix": ".cmake"},
    settings={'cmakelint_config': conf_file})



import re

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY


class DartLintBear(LocalBear, Lint):
    executable = 'dartanalyzer'
    output_regex = re.compile(
        r'\[(?P<severity>error|warning)\] (?P<message>.+)\('
        r'(?P<file_name>.+), line (?P<line>\d+),'
        r' col (?P<column>\d+)\)')
    severity_map = {
        "error": RESULT_SEVERITY.MAJOR,
        "warning": RESULT_SEVERITY.NORMAL}

    def run(self, filename, file):
        '''
        Checks the code with ``dart-linter``.

        This bear expects dart commands to be on your ``PATH``. Please ensure
        /path/to/dart-sdk/bin is in your ``PATH``.
        '''
        self.arguments = '{filename}'
        return self.lint(filename)




from bears.c_languages.ClangBear import ClangBear
from tests.LocalBearTestHelper import verify_local_bear

ClangBearTest = verify_local_bear(
    ClangBear,
    (["int main() {}"], ),
    (["bad things, this is no C code"],  # Has no fixit
     ["struct { int f0; } x = { f0 :1 };"],  # Has a fixit and no range
     ["int main() {int *b; return b}"]),  # Has a fixit and a range
    'test.c')


ClangBearIgnoreTest = verify_local_bear(
    ClangBear,
    # Should ignore the warning, valid!
    (["struct { int f0; } x = { f0 :1 };"],),
    (),
    'test.c',
    settings={'clang_cli_options': '-w'})



from bears.ruby.RuboCopBear import RuboCopBear
from tests.LocalBearTestHelper import verify_local_bear

good_file = """def bad_name
  test if something
end
""".splitlines(keepends=True)

bad_file = """def badName
  if something
    test
    end
end
""".splitlines(keepends=True)


RuboCopBearTest = verify_local_bear(RuboCopBear,
                                    invalid_files=(bad_file,),
                                    valid_files=(good_file,))



import os
from queue import Queue

from bears.java.CheckstyleBear import CheckstyleBear
from tests.BearTestHelper import generate_skip_decorator
from tests.LocalBearTestHelper import LocalBearTestHelper
from coalib.settings.Section import Section


@generate_skip_decorator(CheckstyleBear)
class CheckstyleBearTest(LocalBearTestHelper):

    def setUp(self):
        self.section = Section("test section")
        self.uut = CheckstyleBear(self.section, Queue())
        self.good_file = os.path.join(os.path.dirname(__file__),
                                      "test_files",
                                      "CheckstyleGood.java")
        self.bad_file = os.path.join(os.path.dirname(__file__),
                                     "test_files",
                                     "CheckstyleBad.java")

    def test_run(self):
        self.check_validity(self.uut, [], self.good_file)
        self.check_validity(self.uut, [], self.bad_file, valid=False)



import sys
from bears import Constants

__version__ = Constants.VERSION


def assert_supported_version():  # pragma: no cover
    if not sys.version_info > (3, 2):
        print("coala supports only python 3.3 or later.")
        exit(4)



from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY


class JuliaLintBear(LocalBear, Lint):
    executable = 'julia'
    arguments = '-e \'import Lint.lintfile; lintfile({filename})\''
    output_regex = r'(^.*\.jl):(?P<line>\d+) (?P<severity>.)\d+ (?P<message>.*)'
    use_stdout = True
    severity_map = {
        "E": RESULT_SEVERITY.MAJOR,
        "W": RESULT_SEVERITY.NORMAL,
        "I": RESULT_SEVERITY.INFO
    }

    def run(self, filename, file):
        '''
        Lints Julia code using ``Lint.jl``.
        https://github.com/tonyhffong/Lint.jl
        '''
        return self.lint(filename, file)



import os
import unittest

from bears.c_languages.codeclone_detection import ClangCountingConditions
from bears.c_languages.codeclone_detection.ClangCountVectorCreator import (
    ClangCountVectorCreator)
from tests.c_languages import skip_if_no_clang
from coalib.settings.Setting import Setting


@skip_if_no_clang()
class ClangCountingConditionsTest(unittest.TestCase):

    def setUp(self):
        self.testfile = os.path.abspath(os.path.join(
            os.path.dirname(__file__),
            "conditions_samples.c"))

    def check_counting_condition(self, conditions, function, expected):
        """
        Checks if the given count vectors match the given expected data.

        :param conditions: String indicating which condition(s) to use (will
                           be fed to
                           ClangCountingConditions.counting_condition)
        :param function:   String indicating which function from test file to
                           use. (i.e. "used(int, int)")
        :param expected:   Dict with python lists of counts for all variables.
        """
        counter = ClangCountVectorCreator(
            ClangCountingConditions.counting_condition(
                Setting("irrelevant", conditions)))
        vectors = counter.get_vectors_for_file(self.testfile)

        actual = vectors[function]
        self.assertEqual(len(actual),
                         len(expected),
                         "Actual dict: " + str(actual))
        self.assertEqual(sorted(actual.keys()),
                         sorted(expected.keys()))
        for variable in actual:
            self.assertEqual(actual[variable].count_vector,
                             expected[variable],
                             "Variable '{}' doesnt match.".format(variable))

    def test_conversion(self):
        with self.assertRaises(TypeError):
            ClangCountingConditions.counting_condition(5)

        self.assertEqual(
            ClangCountingConditions.counting_condition(
                Setting("irrelevant", "used, in_condition")),
            [ClangCountingConditions.used,
             ClangCountingConditions.in_condition])

    def test_used(self):
        self.check_counting_condition(
            "used",
            (1, "used(int, int)"),
            {"a": [5],
             "b": [6],
             "foo": [1],
             "0": [1]})

    def test_is_called(self):
        self.check_counting_condition(
            "is_called",
            (1, "used(int, int)"),
            {"a": [0],
             "b": [0],
             "foo": [1],
             "0": [0]})

    def test_is_call_param(self):
        self.check_counting_condition(
            "is_call_param",
            (1, "used(int, int)"),
            {"a": [0],
             "b": [1],
             "foo": [0],
             "0": [0]})

    def test_returned(self):
        self.check_counting_condition(
            "returned",
            (13, "returned(int, int)"),
            {"a": [3],
             "b": [2]})

    def test_is_condition(self):
        self.check_counting_condition(
            "is_condition",
            (22, "loopy(int, int)"),
            {"a": [2],
             "b": [1],
             "0": [0]})

    def test_in_condition(self):
        self.check_counting_condition(
            "in_condition",
            (47, "in_condition(int, int)"),
            {"a": [1],
             "b": [1],
             "c": [1],
             "d": [1]})

        self.check_counting_condition(
            "in_condition",
            (111, "levels(int, int, int)"),
            {"first": [3],
             "second": [0],
             "third": [0],
             "0": [0],
             "1": [2],
             "2": [2],
             "3": [0],
             "5": [0]})

        self.check_counting_condition(
            "in_second_level_condition",
            (111, "levels(int, int, int)"),
            {"first": [0],
             "second": [1],
             "third": [0],
             "0": [0],
             "1": [1],
             "2": [0],
             "3": [1],
             "5": [0]})

        self.check_counting_condition(
            "in_third_level_condition",
            (111, "levels(int, int, int)"),
            {"first": [0],
             "second": [0],
             "third": [2],
             "0": [0],
             "1": [0],
             "2": [1],
             "3": [1],
             "5": [0]})

    def test_is_assignee(self):
        self.check_counting_condition(
            "is_assignee",
            (62, "assignation(int, int)"),
            {"a": [3],
             "b": [9],
             "1": [0]})

    def test_is_assigner(self):
        self.check_counting_condition(
            "is_assigner",
            (62, "assignation(int, int)"),
            {"a": [6],
             "b": [9],
             "1": [4]})

    def test_loop_content(self):
        self.check_counting_condition(
            "loop_content",
            (22, "loopy(int, int)"),
            {"a": [0],
             "b": [6],
             "0": [0]})

        self.check_counting_condition(
            "loop_content",
            (111, "levels(int, int, int)"),
            {"first": [1],
             "second": [0],
             "third": [0],
             "0": [0],
             "1": [2],
             "2": [0],
             "3": [0],
             "5": [0]})

        self.check_counting_condition(
            "second_level_loop_content",
            (111, "levels(int, int, int)"),
            {"first": [0],
             "second": [1],
             "third": [0],
             "0": [0],
             "1": [0],
             "2": [2],
             "3": [0],
             "5": [0]})

        self.check_counting_condition(
            "third_level_loop_content",
            (111, "levels(int, int, int)"),
            {"first": [0],
             "second": [0],
             "third": [2],
             "0": [0],
             "1": [0],
             "2": [0],
             "3": [1],
             "5": [1]})

    def test_is_param(self):
        self.check_counting_condition(
            "is_param",
            (47, "in_condition(int, int)"),
            {"a": [1],
             "b": [1],
             "c": [0],
             "d": [0]})

    def test_in_operation(self):
        self.check_counting_condition(
            "in_sum",
            (89, "arithmetics(int, int)"),
            {"a": [4],
             "b": [3],
             "4": [1]})

        self.check_counting_condition(
            "in_product",
            (89, "arithmetics(int, int)"),
            {"a": [6],
             "b": [6],
             "4": [0]})

        self.check_counting_condition(
            "in_binary_operation",
            (89, "arithmetics(int, int)"),
            {"a": [6],
             "b": [2],
             "4": [0]})

    def test_member_accessed(self):
        self.check_counting_condition(
            "member_accessed",
            (149, "structing(struct test_struct, struct test_struct *)"),
            {"a": [1],
             "b": [1],
             "1": [0],
             "2": [1]})



from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear


class PyDocStyleBear(LocalBear, Lint):
    executable = 'pydocstyle'
    arguments = '{filename}'
    output_regex = r'(.*\.py):(?P<line>\d+) (.+):\n\s+(?P<message>.*)'
    use_stderr = True

    def run(self, filename, file):
        '''
        Checks python docstrings.
        '''
        return self.lint(filename, file)



#!/usr/bin/env python3

# Start ignoring PyImportSortBear as imports below may yield syntax errors
from bears import assert_supported_version
assert_supported_version()
# Stop ignoring

import locale
import sys
from os.path import exists
from shutil import copyfileobj
from subprocess import call
from urllib.request import urlopen

import setuptools.command.build_py
from bears import Constants
from setuptools import find_packages, setup
from setuptools.command.test import test as TestCommand

try:
    locale.getlocale()
except (ValueError, UnicodeError):
    locale.setlocale(locale.LC_ALL, 'en_US.UTF-8')


def download(url, filename, overwrite=False):
    """
    Downloads the given URL to the given filename. If the file exists, it won't
    be downloaded.

    :param url:       A URL to download.
    :param filename:  The file to store the downloaded file to.
    :param overwrite: Set to True if the file should be downloaded even if it
                      already exists.
    :return:          The filename.
    """
    if not exists(filename) or overwrite:
        print("Downloading", filename + "...")
        with urlopen(url) as response, open(filename, 'wb') as out_file:
            copyfileobj(response, out_file)
        print("DONE.")

    return filename


class PyTestCommand(TestCommand):

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main([])
        sys.exit(errno)


class BuildDocsCommand(setuptools.command.build_py.build_py):
    apidoc_command = ('sphinx-apidoc', '-f', '-o', 'docs/API',
                      'bears')
    make_command = ('make', '-C', 'docs', 'html', 'SPHINXOPTS=-W')

    def run(self):
        call(self.apidoc_command)
        call(self.make_command)


with open('requirements.txt') as requirements:
    required = requirements.read().splitlines()

with open('test-requirements.txt') as requirements:
    test_required = requirements.read().splitlines()

with open("README.rst") as readme:
    long_description = readme.read()


if __name__ == "__main__":
    download('http://sourceforge.net/projects/checkstyle/files/checkstyle/'
             '6.15/checkstyle-6.15-all.jar',
             'bears/java/checkstyle.jar')

    download('https://oss.sonatype.org/content/repositories/releases/org/'
             'scalastyle/scalastyle_2.10/0.8.0/scalastyle_2.10-0.8.0-batch.jar',
             'bears/scala/scalastyle.jar')

    setup(name='coala-bears',
          version=Constants.VERSION,
          description='Bears for coala (Code Analysis Application)',
          author="The coala developers",
          maintainer="Lasse Schuirmann, Fabian Neuschmidt, Mischa Kr\xfcger",
          maintainer_email=('lasse.schuirmann@gmail.com, '
                            'fabian@neuschmidt.de, '
                            'makman@alice.de'),
          url='http://coala.rtfd.org/',
          platforms='any',
          packages=find_packages(exclude=["build.*", "*.tests.*", "*.tests"]),
          install_requires=required,
          tests_require=test_required,
          package_data={'bears': ["VERSION"],
                        'bears.java': ['checkstyle.jar', 'google_checks.xml'],
                        'bears.scala': ['scalastyle.jar',
                                        'scalastyle_config.xml']},
          license="AGPL-3.0",
          long_description=long_description,
          entry_points={"coalabears": ["coala_official_bears = bears"]},
          # from http://pypi.python.org/pypi?%3Aaction=list_classifiers
          classifiers=[
              'Development Status :: 4 - Beta',

              'Environment :: Console',
              'Environment :: MacOS X',
              'Environment :: Win32 (MS Windows)',
              'Environment :: X11 Applications :: Gnome',

              'Intended Audience :: Science/Research',
              'Intended Audience :: Developers',

              'License :: OSI Approved :: GNU Affero General Public License '
              'v3 or later (AGPLv3+)',

              'Operating System :: OS Independent',

              'Programming Language :: Python :: Implementation :: CPython',
              'Programming Language :: Python :: 3.3',
              'Programming Language :: Python :: 3.4',
              'Programming Language :: Python :: 3.5',
              'Programming Language :: Python :: 3 :: Only',

              'Topic :: Scientific/Engineering :: Information Analysis',
              'Topic :: Software Development :: Quality Assurance',
              'Topic :: Text Processing :: Linguistic'],
          cmdclass={'docs': BuildDocsCommand,
                    'test': PyTestCommand})



import os
import platform
import re
import shutil
import stat
import unittest
from queue import Queue
from tempfile import mkdtemp

from tests.BearTestHelper import generate_skip_decorator
from bears.vcs.git.GitCommitBear import GitCommitBear
from coalib.misc.Shell import run_shell_command
from coalib.settings.ConfigurationGathering import get_config_directory
from coalib.settings.Section import Section
from coalib.settings.Setting import Setting


@generate_skip_decorator(GitCommitBear)
class GitCommitBearTest(unittest.TestCase):

    @staticmethod
    def run_git_command(*args, stdin=None):
        run_shell_command(" ".join(("git",) + args), stdin)

    @staticmethod
    def git_commit(msg):
        # Use stdin mode from git, since -m on Windows cmd does not support
        # multiline messages.
        GitCommitBearTest.run_git_command("commit",
                                          "--allow-empty",
                                          "--allow-empty-message",
                                          "--file=-",
                                          stdin=msg)

    def run_uut(self, *args, **kwargs):
        """
        Runs the unit-under-test (via `self.uut.run()`) and collects the
        messages of the yielded results as a list.

        :param args:   Positional arguments to forward to the run function.
        :param kwargs: Keyword arguments to forward to the run function.
        :return:       A list of the message strings.
        """
        return list(result.message for result in self.uut.run(*args, **kwargs))

    def setUp(self):
        self.msg_queue = Queue()
        self.section = Section("")
        self.uut = GitCommitBear(None, self.section, self.msg_queue)

        self._old_cwd = os.getcwd()
        self.gitdir = mkdtemp()
        os.chdir(self.gitdir)
        self.run_git_command("init")
        self.run_git_command("config", "user.email coala@coala-analyzer.io")
        self.run_git_command("config", "user.name coala")

    @staticmethod
    def _windows_rmtree_remove_readonly(func, path, excinfo):
        os.chmod(path, stat.S_IWRITE)
        func(path)

    def tearDown(self):
        os.chdir(self._old_cwd)
        if platform.system() == "Windows":
            onerror = self._windows_rmtree_remove_readonly
        else:
            onerror = None
        shutil.rmtree(self.gitdir, onerror=onerror)

    def test_check_prerequisites(self):
        _shutil_which = shutil.which
        try:
            shutil.which = lambda *args, **kwargs: None
            self.assertEqual(GitCommitBear.check_prerequisites(),
                             "git is not installed.")

            shutil.which = lambda *args, **kwargs: "path/to/git"
            self.assertTrue(GitCommitBear.check_prerequisites())
        finally:
            shutil.which = _shutil_which

    def test_git_failure(self):
        # In this case use a reference to a non-existing commit, so just try
        # to log all commits on a newly created repository.
        self.assertEqual(self.run_uut(), [])

        git_error = self.msg_queue.get().message
        self.assertEqual(git_error[:4], "git:")

        self.assertTrue(self.msg_queue.empty())

    def test_empty_message(self):
        self.git_commit("")

        self.assertEqual(self.run_uut(),
                         ["HEAD commit has no message."])
        self.assertTrue(self.msg_queue.empty())

        self.assertEqual(self.run_uut(allow_empty_commit_message=True),
                         [])
        self.assertTrue(self.msg_queue.empty())

    def test_shortlog_checks_length(self):
        self.git_commit("Commits messages that nearly exceeds default limit")

        self.assertEqual(self.run_uut(), [])
        self.assertTrue(self.msg_queue.empty())

        self.assertEqual(self.run_uut(shortlog_length=17),
                         ["Shortlog of HEAD commit is too long."])
        self.assertTrue(self.msg_queue.empty())

        self.git_commit("A shortlog that is too long is not good for history")
        self.assertEqual(self.run_uut(),
                         ["Shortlog of HEAD commit is too long."])
        self.assertTrue(self.msg_queue.empty())

    def test_shortlog_checks_shortlog_trailing_period(self):
        self.git_commit("Shortlog with dot.")
        self.assertEqual(self.run_uut(shortlog_trailing_period=True), [])
        self.assertEqual(self.run_uut(shortlog_trailing_period=False),
                         ["Shortlog of HEAD commit contains a period at end."])
        self.assertEqual(self.run_uut(shortlog_trailing_period=None), [])

        self.git_commit("Shortlog without dot")
        self.assertEqual(
            self.run_uut(shortlog_trailing_period=True),
            ["Shortlog of HEAD commit contains no period at end."])
        self.assertEqual(self.run_uut(shortlog_trailing_period=False), [])
        self.assertEqual(self.run_uut(shortlog_trailing_period=None), [])

    def test_shortlog_checks_regex(self):
        pattern = ".*?: .*[^.]"

        self.git_commit("tag: message")
        self.assertEqual(self.run_uut(shortlog_regex=pattern), [])

        self.git_commit("tag: message invalid.")
        self.assertEqual(
            self.run_uut(shortlog_regex=pattern),
            ["Shortlog of HEAD commit does not match given regex."])

        self.git_commit("SuCkS cOmPleTely")
        self.assertEqual(
            self.run_uut(shortlog_regex=pattern),
            ["Shortlog of HEAD commit does not match given regex."])

        # Check for full-matching.
        pattern = "abcdefg"

        self.git_commit("abcdefg")
        self.assertEqual(self.run_uut(shortlog_regex=pattern), [])

        self.git_commit("abcdefgNO MATCH")
        self.assertEqual(
            self.run_uut(shortlog_regex=pattern),
            ["Shortlog of HEAD commit does not match given regex."])

    def test_body_checks(self):
        self.git_commit(
            "Commits message with a body\n\n"
            "nearly exceeding the default length of a body, but not quite. "
            "haaaaaaands")

        self.assertEqual(self.run_uut(), [])
        self.assertTrue(self.msg_queue.empty())

        self.git_commit("Shortlog only")

        self.assertEqual(self.run_uut(), [])
        self.assertTrue(self.msg_queue.empty())

        # Force a body.
        self.git_commit("Shortlog only ...")
        self.assertEqual(self.run_uut(force_body=True),
                         ["No commit message body at HEAD."])
        self.assertTrue(self.msg_queue.empty())

        # Miss a newline between shortlog and body.
        self.git_commit("Shortlog\nOops, body too early")
        self.assertEqual(self.run_uut(),
                         ["No newline between shortlog and body at HEAD."])
        self.assertTrue(self.msg_queue.empty())

        # And now too long lines.
        self.git_commit("Shortlog\n\n"
                        "This line is ok.\n"
                        "This line is by far too long (in this case).\n"
                        "This one too, blablablablablablablablabla.")
        self.assertEqual(self.run_uut(body_line_length=41),
                         ["Body of HEAD commit contains too long lines."])
        self.assertTrue(self.msg_queue.empty())

    def test_different_path(self):
        no_git_dir = mkdtemp()
        self.git_commit("A shortlog that is too long is not good for history")
        os.chdir(no_git_dir)
        # When section doesn't have a config setting
        self.assertEqual(self.run_uut(), [])
        git_error = self.msg_queue.get().message
        self.assertEqual(git_error[:4], "git:")
        # when section does have a config setting
        self.section.append(Setting("config", re.escape(self.gitdir)))
        self.assertEqual(self.run_uut(),
                         ["Shortlog of HEAD commit is too long."])
        self.assertEqual(get_config_directory(self.section),
                         self.gitdir)
        os.chdir(self.gitdir)
        os.rmdir(no_git_dir)



import json
import re

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear


class MarkdownBear(Lint, LocalBear):
    executable = 'remark'
    diff_message = "The text does not comply to the set style."
    arguments = '--no-color --quiet'
    gives_corrected = True
    use_stdin = True

    def run(self, filename, file,
            markdown_bullets: str="-",
            markdown_closed_headings: bool=False,
            markdown_setext_headings: bool=False,
            markdown_emphasis: str="*",
            markdown_strong: str="*",
            markdown_encode_entities: bool=False,
            markdown_codefence: str="`",
            markdown_fences: bool=True,
            markdown_list_indent: str="1",
            markdown_loose_tables: bool=False,
            markdown_spaced_tables: bool=True,
            markdown_list_increment: bool=True,
            markdown_horizontal_rule: str='*',
            markdown_horizontal_rule_spaces: bool=False,
            markdown_horizontal_rule_repeat: int=3):
        """
        Raises issues against style violations on markdown files.

        :param markdown_bullets:                Character to use for bullets
                                                in lists. Can be "-", "*"
                                                or "+".
        :param markdown_closed_headings:        Whether to close Atx headings
                                                or not. if true, extra # marks
                                                will be required after the
                                                heading. eg: `## Heading ##`.
        :param markdown_setext_headings:        Whether to use setext headings.
                                                A setext heading uses
                                                underlines instead of # marks.
        :param markdown_emphasis:               Character to wrap strong
                                                emphasis by. Can be "_" or "*".
        :param markdown_strong:                 Character to wrap slight
                                                emphasis by. Can be "_" or "*".
        :param markdown_encode_entities:        Whether to encode symbols that
                                                are not ASCII into special
                                                HTML characters.
        :param markdown_codefence:              Used to find which characters
                                                to use for code fences.
                                                Can be "`" or "~".
        :param markdown_fences:                 Use fences for code blocks.
        :param markdown_list_indent:            Used to find spacing after
                                                bullet in lists.
                                                Can be "1", "tab" or "mixed"

                                                - "1" - 1 space after bullet.
                                                - "tab" - Use tab stops to
                                                  begin a sentence after the
                                                  bullet.
                                                - "mixed" - Use "1" when the
                                                  list item is only 1 line,
                                                  "tab" if it spans multiple.
        :param markdown_loose_tables:           Whether to use pipes for the
                                                outermost borders in a table.
        :param markdown_spaced_tables:          Whether to add space between
                                                pipes in a table.
        :param markdown_list_increment:         Whether an ordered lists
                                                numbers should be incremented.
        :param markdown_horizontal_rule:        The horizontal rule character.
                                                Can be '*', '_' or '-'.
        :param markdown_horizontal_rule_spaces: Whether spaces should be added
                                                between horizontal rule
                                                characters.
        :param markdown_horizontal_rule_repeat: The number of times the
                                                horizontal rule character will
                                                be repeated.
        """
        self.remark_configs = {
            "bullet": markdown_bullets,                         # - or *
            "closeAtx": markdown_closed_headings,               # Bool
            "setext": markdown_setext_headings,                 # Bool
            "emphasis": markdown_emphasis,                      # char (_ or *)
            "strong": markdown_strong,                          # char (_ or *)
            "entities": markdown_encode_entities,               # Bool
            "fence": markdown_codefence,                        # char (~ or `)
            "fences": markdown_fences,                          # Bool
            "listItemIndent": markdown_list_indent,             # int or "tab"
                                                                # or "mixed"
            "looseTable": markdown_loose_tables,                # Bool
            "spacedTable": markdown_spaced_tables,              # Bool
            "incrementListMarker": markdown_list_increment,     # Bool
            "rule": markdown_horizontal_rule,                   # - or * or _
            "ruleSpaces": markdown_horizontal_rule_spaces,      # Bool
            "ruleRepetition": markdown_horizontal_rule_repeat,  # int
        }
        config_json = json.dumps(self.remark_configs)
        # Remove { and } (remark adds it on its own) and escape it
        settings = re.escape(config_json[1:-1])
        self.arguments += " --setting " + settings
        return self.lint(filename, file)



from bears.yaml.YAMLLintBear import YAMLLintBear
from coalib.misc.ContextManagers import prepare_file
from tests.LocalBearTestHelper import verify_local_bear

test_file = """
---
receipt:     Oz-Ware Purchase Invoice
date:        2012-08-06
customer:
    first_name:   Dorothy
    family_name:  Gale

items:
    - part_no:   A4786
      descrip:   Water Bucket (Filled)
      price:     1.47
      quantity:  4
...
""".splitlines(keepends=True)

config_file = """
extends:
    default
rules:
    colons:
      max-spaces-after: -1
    indentation: disable
    empty-lines: disable
""".splitlines(keepends=True)


YAMLLintBear1Test = verify_local_bear(YAMLLintBear,
                                      valid_files=(),
                                      invalid_files=(test_file,))

with prepare_file(config_file,
                  filename=None,
                  force_linebreaks=True,
                  create_tempfile=True) as (conf_lines, conf_file):
    YAMLLintBear2Test = verify_local_bear(YAMLLintBear,
                                          valid_files=(test_file,),
                                          invalid_files=(),
                                          settings={
                                              'yamllint_config': conf_file})



from bears.markdown.MarkdownBear import MarkdownBear
from tests.LocalBearTestHelper import verify_local_bear

test_file1 = """1. abc
1. def
""".splitlines(keepends=True)


test_file2 = """1. abc
2. def
""".splitlines(keepends=True)


MarkdownBearTest = verify_local_bear(MarkdownBear,
                                     valid_files=(test_file2,),
                                     invalid_files=(test_file1,))

MarkdownBearConfigsTest = verify_local_bear(
    MarkdownBear,
    valid_files=(test_file1,),
    invalid_files=(test_file2,),
    settings={"markdown_list_increment": False})



from bears.julia.JuliaLintBear import JuliaLintBear
from tests.LocalBearTestHelper import verify_local_bear

good_file = """
a = 2
println(2)
""".splitlines(keepends=True)

bad_file = """
println(hello)
""".splitlines(keepends=True)

JuliaLintBearTest = verify_local_bear(JuliaLintBear,
                                      valid_files=(good_file,),
                                      invalid_files=(bad_file,),
                                      tempfile_kwargs={"suffix": ".jl"},
                                      timeout=45)



import shutil
import unittest
from unittest.case import SkipTest

from bears.natural_language.LanguageToolBear import LanguageToolBear
from tests.BearTestHelper import generate_skip_decorator
from tests.LocalBearTestHelper import verify_local_bear

try:
    import language_check
    import guess_language
except ImportError as err:
    raise SkipTest(str(err))


LanguageToolBearTest = verify_local_bear(
    LanguageToolBear,
    valid_files=(["A correct English sentence sounds nice in everyone."],
                 ["Eine korrekte englische Satz klingt nett zu jedermann."]),
    invalid_files=(["  "],
                   ["asdgaasdfgahsadf"],
                   ['"quoted"']))


LanguageToolBearLocaleTest = verify_local_bear(
    LanguageToolBear,
    valid_files=(["A correct English sentence sounds nice in everyone."],),
    invalid_files=(["Eine korrekte englische Satz klingt nett zu jedermann."],),
    settings={'locale': 'en-US'})


LanguageToolBearDisableRulesTest = verify_local_bear(
    LanguageToolBear,
    valid_files=(["Line without unnecessary spaces at the start."],
                 ["a line beginning with lowercase."],
                 ["A line beginning with uppercase."]),
    invalid_files=(["  Line with unnecessary spaces at the start."],),
    settings={'languagetool_disable_rules': 'UPPERCASE_SENTENCE_START'})


@generate_skip_decorator(LanguageToolBear)
class LanguageToolBearPrerequisitesTest(unittest.TestCase):

    def test_check_prerequisites(self):
        _shutil_which = shutil.which
        try:
            shutil.which = lambda *args, **kwargs: None
            self.assertEqual(LanguageToolBear.check_prerequisites(),
                             "java is not installed.")

            shutil.which = lambda *args, **kwargs: "path/to/java"
            self.assertTrue(LanguageToolBear.check_prerequisites())
        finally:
            shutil.which = _shutil_which



from bears.matlab.MatlabIndentationBear import MatlabIndentationBear
from tests.LocalBearTestHelper import verify_local_bear

MatlabIndentationBearTest = verify_local_bear(
    MatlabIndentationBear,
    valid_files=(["if a ~= b\n", "  a\n", "endif\n"],
                 ("if a ~= b\n", "  a\n", "endif\n"),
                 ["if a ~= b\n",
                  "  a\n",
                  "  \n",
                  "else\n",
                  "  a\n",
                  "endif\n"]),
    invalid_files=(["  A"],
                   ["if a ~= b\n", "a\n", "endif\n"],
                   ["if a ~= b\n", " a\n", "endif\n"],
                   ("if a ~= b\n", " a\n", "endif\n"),
                   ["if a ~= b\n",
                    "  a\n",
                    "  else\n",
                    "  a\n",
                    "endif\n"]))



import re

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY


class LuaLintBear(LocalBear, Lint):
    executable = 'luacheck'
    output_regex = re.compile(
        r'\s*(?P<filename>.+):(?P<line>\d+):'
        r'(?P<column>\d+):\s(?P<message>.+)')

    def run(self, filename, file):
        '''
        Checks the code with ``luacheck``.
        '''
        self.arguments = '{filename}'
        return self.lint(filename)



import re

from coalib.bearlib.abstractions.Lint import Lint
from coalib.bears.LocalBear import LocalBear
from coalib.results.RESULT_SEVERITY import RESULT_SEVERITY


class RLintBear(LocalBear, Lint):
    executable = 'Rscript'
    arguments = "-e 'library(lintr)' -e 'lintr::lint({filename})'"
    output_regex = re.compile(
        r'(?P<file_name>.*?):(?P<line>\d+):(?P<column>\d+):'
        r' (?P<severity>\S+): (?P<message>.*)')
    severity_map = {
        "style": RESULT_SEVERITY.NORMAL,
        "warning": RESULT_SEVERITY.NORMAL,
        "error": RESULT_SEVERITY.MAJOR}

    prerequisite_command = ["Rscript", "-e", "library(lintr)"]
    prerequisite_fail_msg = 'R library "lintr" is not installed.'

    def run(self, filename, file):
        '''
        Checks the code with `lintr`.
        '''
        return self.lint(filename)



from bears.natural_language.reSTLintBear import reSTLintBear
from tests.LocalBearTestHelper import verify_local_bear

good_file = ["test\n====\n"]
bad_file = ["test\n==\n"]


reSTLintBearTest = verify_local_bear(reSTLintBear,
                                     valid_files=(good_file,),
                                     invalid_files=(bad_file,))




from bears.go.GoImportsBear import GoImportsBear
from tests.LocalBearTestHelper import verify_local_bear

GoImportsBearTest = verify_local_bear(
    GoImportsBear,
    (['package main', '', 'import "os"', '',
      'func main() {', '\tf, _ := os.Open("foo")', '}'],),
    (['package main', '', '',
      'func main() {', '\tf, _ := os.Open("foo")', '}'],))
